from abjad.tools import durationtools
from abjad.tools.beamtools.BeamSpanner.BeamSpanner import BeamSpanner


class ComplexBeamSpanner(BeamSpanner):
    r'''Abjad complex beam spanner::

        >>> staff = Staff("c'16 e'16 r16 f'16 g'2")

    ::

        >>> f(staff)
        \new Staff {
            c'16
            e'16
            r16
            f'16
            g'2
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.ComplexBeamSpanner(staff[:4])
        ComplexBeamSpanner(c'16, e'16, r16, f'16)

    ::

        >>> f(staff)
        \new Staff {
            \set stemLeftBeamCount = #0
            \set stemRightBeamCount = #2
            c'16 [
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #2
            e'16 ]
            r16
            \set stemLeftBeamCount = #2
            \set stemRightBeamCount = #0
            f'16 [ ]
            g'2
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return complex beam spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, lone=False, direction=None):
        BeamSpanner.__init__(self, components=components, direction=direction)
        self.lone = lone

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        BeamSpanner._copy_keyword_args(self, new)
        new.lone = self.lone

    def _format_before_leaf(self, leaf):
        '''Spanner format contribution to output before leaf.'''
        from abjad.tools import beamtools
        result = []
        result.extend(BeamSpanner._format_before_leaf(self, leaf))
        if beamtools.is_beamable_component(leaf):
            if self._is_my_only_leaf(leaf):
                left, right = self._get_left_right_for_lone_leaf(leaf)
            elif self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    def _format_right_of_leaf(self, leaf):
        '''Spanner format contribution to output right of leaf.'''
        from abjad.tools import beamtools
        from abjad.tools import leaftools
        result = []
        #if leaf.beam.beamable:
        if beamtools.is_beamable_component(leaf):
            previous_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, -1)
            next_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1)
            # lone
            if self._is_my_only_leaf(leaf):
                if self.lone:
                    if self.direction is not None:
                        result.append('%s [' % self.direction)
                    else:
                        result.append('[')
            # otherwise
            elif self._is_my_first_leaf(leaf) or not previous_leaf or \
                not beamtools.is_beamable_component(previous_leaf):
                if self.direction is not None:
                    result.append('%s [' % self.direction)
                else:
                    result.append('[')
            # lone
            if self._is_my_only_leaf(leaf):
                if self.lone:
                    result.append(']')
            # otherwise
            elif self._is_my_last_leaf(leaf) or not next_leaf or \
                not beamtools.is_beamable_component(next_leaf):
                result.append(']')
        return result

    def _get_left_right_for_exterior_leaf(self, leaf):
        '''Get left and right flag counts for exterior leaf in spanner.'''
        from abjad.tools import leaftools
        # lone
        if self._is_my_only_leaf(leaf):
            left, right = self._get_left_right_for_lone_leaf(leaf)
        # first
        elif self._is_my_first_leaf(leaf) or not leaftools.get_nth_leaf_in_thread_from_leaf(leaf, -1):
            left = 0
            right = leaf.written_duration.flag_count
        # last
        elif self._is_my_last_leaf(leaf) or not leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1):
            left = leaf.written_duration.flag_count
            right = 0
        else:
            raise ValueError('leaf must be first or last in spanner.')
        return left, right

    def _get_left_right_for_interior_leaf(self, leaf):
        '''Interior leaves are neither first nor last in spanner.
        Interior leaves may be surrounded by beamable leaves.
        Interior leaves may be surrounded by unbeamable leaves.
        Four cases total for beamability of surrounding leaves.'''
        from abjad.tools import beamtools
        from abjad.tools import leaftools
        previous_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, -1)
        previous_written = previous_leaf.written_duration
        current_written = leaf.written_duration
        next_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1)
        next_written = next_leaf.written_duration
        previous_flag_count = previous_written.flag_count
        current_flag_count = current_written.flag_count
        next_flag_count = next_written.flag_count
        # [unbeamable leaf beamable]
        if not beamtools.is_beamable_component(previous_leaf) and \
            beamtools.is_beamable_component(next_leaf):
            left = current_flag_count
            right = min(current_flag_count, next_flag_count)
        # [beamable leaf unbeamable]
        if beamtools.is_beamable_component(previous_leaf) and \
            not beamtools.is_beamable_component(next_leaf):
            left = min(current_flag_count, previous_flag_count)
            right = current_flag_count
        # [unbeamable leaf unbeamable]
        elif not beamtools.is_beamable_component(previous_leaf) and \
            not beamtools.is_beamable_component(next_leaf):
            left = current_flag_count
            right = current_flag_count
        # [beamable leaf beamable]
        else:
            left = min(current_flag_count, previous_flag_count)
            right = min(current_flag_count, next_flag_count)
            if left != current_flag_count and right != current_flag_count:
                left = current_flag_count
        return left, right

    def _get_left_right_for_lone_leaf(self, leaf):
        '''Get left and right flag counts for only leaf in spanner.'''
        current_flag_count = leaf.written_duration.flag_count
        left, right = None, None
        if self.lone == 'left':
            left = current_flag_count
            right = 0
        elif self.lone == 'right':
            left = 0
            right = current_flag_count
        elif self.lone in ('both', True):
            left = current_flag_count
            right = current_flag_count
        elif self.lone in ('neither', False):
            left = None
            right = None
        else:
            raise ValueError("'lone' must be left, right, both.")
        return left, right

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def lone():
        def fget(self):
            r'''Beam lone leaf and force beam nibs to left::

                >>> note = Note("c'16")

            ::

                >>> beam = beamtools.ComplexBeamSpanner([note], lone='left')

            ::

                >>> f(note)
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                c'16 [ ]

            Beam lone leaf and force beam nibs to right::

                >>> note = Note("c'16")

            ::

                >>> beam = beamtools.ComplexBeamSpanner([note], lone='right')

            ::

                >>> f(note)
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [ ]

            Beam lone leaf and force beam nibs to both left and right::

                >>> note = Note("c'16")

            ::

                >>> beam = beamtools.ComplexBeamSpanner([note], lone='both')

            ::

                >>> f(note)
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #2
                c'16 [ ]

            Beam lone leaf and accept LilyPond default nibs at both left and right::

                >>> note = Note("c'16")

            ::

                >>> beam = beamtools.ComplexBeamSpanner([note], lone=True)

            ::

                >>> f(note)
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #2
                c'16 [ ]

            Do not beam lone leaf::

                >>> note = Note("c'16")

            ::

                >>> beam = beamtools.ComplexBeamSpanner([note], lone=False)

            ::

                >>> f(note)
                c'16

            Set to ``'left'``, ``'right'``, ``'both'``, true or false as shown above.

            Ignore this setting when spanner contains more than one leaf.
            '''
            return self._lone
        def fset(self, arg):
            assert isinstance(arg, bool) or arg in ('left', 'right', 'both')
            self._lone = arg
        return property(**locals())



import abc
from abjad.tools import documentationtools
from abjad.tools import iotools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
from abjad.tools.developerscripttools.ReplaceInFilesScript import ReplaceInFilesScript
import os


class RenameModulesScript(DeveloperScript):
    '''Rename classes and functions.

    Handle renaming the module and package, as well as any tests, documentation or
    mentions of the class throughout the Abjad codebase:

    ::

        $ ajv rename -h
        usage: rename-modules [-h] [--version] (-C | -F)

        Rename public modules.

        optional arguments:
          -h, --help       show this help message and exit
          --version        show program's version number and exit
          -C, --classes    rename classes
          -F, --functions  rename functions

    Return `RenameModulesScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'rename'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'Rename public modules.'

    @property
    def version(self):
        return 1.0

    ### PRIVATE METHODS ###

    def _codebase_name_to_codebase_docs_path(self, codebase):
        from abjad import ABJCFG
        if codebase == 'mainline':
            return os.path.join(ABJCFG.ABJAD_PATH, 'docs', 'source', 'api', 'tools')
        elif codebase == 'experimental':
            return os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'docs', 'source', 'tools')
        raise Exception('Bad codebase name {!r}.'.format(codebase))

    def _codebase_name_to_codebase_tools_path(self, codebase):
        from abjad import ABJCFG
        if codebase == 'mainline':
            return os.path.join(ABJCFG.ABJAD_PATH, 'tools')
        elif codebase == 'experimental':
            return os.path.join(ABJCFG.ABJAD_EXPERIMENTAL_PATH, 'tools')
        raise Exception('Bad codebase name {!r}.'.format(codebase))

    def _confirm_name_changes(self, kind, old_codebase, old_package_name, old_object_name,
        new_codebase, new_package_name, new_object_name):

        max_codebase = max(len(old_codebase), len(new_codebase))
        old_codebase = old_codebase.ljust(max_codebase)
        new_codebase = new_codebase.ljust(max_codebase)

        print ''
        print 'Is ...'
        print ''
        print '    [{}] {}.{}()'.format(old_codebase, old_package_name, old_object_name)
        print '    ===>'
        print '    [{}] {}.{}()'.format(new_codebase, new_package_name, new_object_name)
        print ''
        input = raw_input('... correct [Y(es)/n(o)]? ').lower()
        print ''
        if input in ('y', 'yes'):
            return
        else:
            raise SystemExit

    def _get_object_names(self, kind, codebase, tools_package_name):
        assert kind in ('class', 'function')
        tools_path = self._codebase_name_to_codebase_tools_path(codebase)
        path = os.path.join(tools_path, tools_package_name)
        if kind == 'class':
            crawler = documentationtools.ClassCrawler(path, include_private_objects=True)
        elif kind == 'function':
            crawler = documentationtools.FunctionCrawler(path, include_private_objects=True)
        objs = crawler()
        return tuple(sorted([x.__name__ for x in objs]))

    def _get_tools_package_names(self, codebase):
        tools_path = self._codebase_name_to_codebase_tools_path(codebase)
        names = []
        for x in os.listdir(tools_path):
            if os.path.isdir(os.path.join(tools_path, x)):
                if not x.startswith(('_', '.')):
                    names.append(x)
        return tuple(sorted(names))

    def _prompt_for_new_name(self, kind):

        assert kind in ('class', 'function')

        while True:
            message = 'Select codebase ([m]ainline, e[x]perimental):     '
            if kind == 'function':
                message += '   '
            codebase = raw_input(message).lower()
            if codebase in ('mainline', 'experimental'):
                break
            elif codebase == 'm':
                codebase = 'mainline'
                break
            elif codebase == 'x':
                codebase = 'experimental'
                break
            print ''
            print 'Error: invalid codebase selection {!r}.'.format(codebase)
            print ''

        while True:
            message = 'Enter new tools package-qualified {} name:     '.format(kind)
            qualified_object_name = raw_input(message)
            if qualified_object_name.count('.') != 1:
                print ''
                print 'Input not in toolspackage.{} format: {!r}'.format(kind, qualified_object_name)
                print ''
                continue
            qualified_object_name_parts = qualified_object_name.split('.')
            package_name = qualified_object_name_parts[0].lower()
            object_name = qualified_object_name_parts[1]
            if package_name not in self._get_tools_package_names(codebase):
                print ''
                print 'Error: can not find {!r} in {!r} tools packages.'.format(package_name, codebase)
                print ''
                continue
            if object_name in self._get_object_names(kind, codebase, package_name):
                print ''
                print 'Error: the {} {!r} already exists in {!r}.'.format(kind, object_name, package_name)
                print ''
                continue
            return codebase, package_name, object_name

    def _prompt_for_old_name(self, kind):

        assert kind in ('class', 'function')

        while True:
            message = 'Select codebase ([m]ainline, e[x]perimental):     '
            if kind == 'function':
                message += '   '
            codebase = raw_input(message).lower()
            if codebase in ('mainline', 'experimental'):
                break
            elif codebase == 'm':
                codebase = 'mainline'
                break
            elif codebase == 'x':
                codebase = 'experimental'
                break
            print ''
            print 'Error: invalid codebase selection {!r}.'.format(codebase)
            print ''

        while True:
            message = 'Enter current tools package-qualified {} name: '.format(kind)
            qualified_object_name = raw_input(message)
            if qualified_object_name.count('.') != 1:
                print ''
                print 'Input not in toolspackage.{} format: {!r}'.format(kind, qualified_object_name)
                print ''
                continue
            qualified_object_name_parts = qualified_object_name.split('.')
            package_name = qualified_object_name_parts[0].lower()
            object_name = qualified_object_name_parts[1]
            if package_name not in self._get_tools_package_names(codebase):
                print ''
                print 'Error: can not find {!r} in {!r} tools packages.'.format(package_name, codebase)
                print ''
                continue
            if object_name not in self._get_object_names(kind, codebase, package_name):
                print ''
                print 'Error: can not find {!r} in {!r}.'.format(object_name, package_name)
                print ''
                continue
            return codebase, package_name, object_name

    def _rename_old_api_page(self, kind, old_codebase, old_package_name, old_object_name,
        new_codebase, new_package_name, new_object_name):

        print 'Renaming old API page ...'

        old_docs_path = self._codebase_name_to_codebase_docs_path(old_codebase)
        new_docs_path = self._codebase_name_to_codebase_docs_path(new_codebase)

        old_rst_file_name = old_object_name + '.rst'
        new_rst_file_name = new_object_name + '.rst'

        if kind == 'function':
            old_api_path = os.path.join(old_docs_path, old_package_name, old_rst_file_name)
            new_api_path = os.path.join(new_docs_path, new_package_name, new_rst_file_name)
            command = 'svn --force mv {} {} > /dev/null'.format(old_api_path, new_api_path)
            iotools.spawn_subprocess(command)

        elif kind == 'class':
            # move the folder...
            old_api_path = os.path.join(old_docs_path, old_package_name, old_object_name)
            new_api_path = os.path.join(new_docs_path, new_package_name, new_object_name)
            command = 'svn --force mv {} {} > /dev/null'.format(old_api_path, new_api_path)
            iotools.spawn_subprocess(command)

            # ...then the file
            old_rst_file_name = os.path.join(new_api_path, old_rst_file_name)
            new_rst_file_name = os.path.join(new_api_path, new_rst_file_name)
            command = 'svn --force mv {} {} > /dev/null'.format(old_rst_file_name, new_rst_file_name)
            iotools.spawn_subprocess(command)

        print ''

    def _rename_old_module(self, kind, old_codebase, old_package_name, old_object_name,
        new_codebase, new_package_name, new_object_name):

        print 'Renaming old module ...'

        old_tools_path = self._codebase_name_to_codebase_tools_path(old_codebase)
        new_tools_path = self._codebase_name_to_codebase_tools_path(new_codebase)

        if kind == 'function':
            old_module = old_object_name + '.py'
            old_path = os.path.join(old_tools_path, old_package_name, old_module)
            new_module = new_object_name + '.py'
            new_path = os.path.join(new_tools_path, new_package_name, new_module)
            command = 'svn --force mv {} {} > /dev/null'.format(old_path, new_path)
            iotools.spawn_subprocess(command)

        elif kind == 'class':
            # move the folder...
            old_class_package = os.path.join(old_tools_path, old_package_name, old_object_name)
            new_class_package = os.path.join(new_tools_path, new_package_name, new_object_name)
            command = 'svn --force mv {} {} > /dev/null'.format(old_class_package, new_class_package)
            iotools.spawn_subprocess(command)

            # ...then the file
            old_class_module = os.path.join(new_class_package, '{}.py'.format(old_object_name))
            new_class_module = os.path.join(new_class_package, '{}.py'.format(new_object_name))
            command = 'svn --force mv {} {} > /dev/null'.format(old_class_module, new_class_module)
            iotools.spawn_subprocess(command)

        print ''

    def _rename_old_test_files(self, kind, old_codebase, old_package_name, old_object_name,
        new_codebase, new_package_name, new_object_name):

        print 'Renaming old test file(s) ...'

        old_tools_path = self._codebase_name_to_codebase_tools_path(old_codebase)
        new_tools_path = self._codebase_name_to_codebase_tools_path(new_codebase)

        if kind == 'function':
            old_test_file = 'test_{}_{}.py'.format(old_package_name, old_object_name)
            old_path = os.path.join(old_tools_path, old_package_name, 'test')
            old_path = os.path.join(old_path, old_test_file)
            new_test_file = 'test_{}_{}.py'.format(new_package_name, new_object_name)
            new_path = os.path.join(new_tools_path, new_package_name, 'test')
            new_path = os.path.join(new_path, new_test_file)
            command = 'svn --force mv {} {} > /dev/null'.format(old_path, new_path)
            iotools.spawn_subprocess(command)

        elif kind == 'class':
            test_path = os.path.join(new_tools_path, new_package_name, new_object_name, 'test')
            for x in os.listdir(test_path):
                if x.startswith('test_') and x.endswith('.py'):
                    old_path = os.path.join(test_path, x)
                    new_path = os.path.join(test_path,
                        x.replace('test_{}'.format(old_object_name), 'test_{}'.format(new_object_name)))
                    command = 'svn --force mv {} {} > /dev/null'.format(old_path, new_path)
                    iotools.spawn_subprocess(command)

        print ''

    def _update_codebase(self, kind, old_codebase, old_package_name, old_object_name,
        new_codebase, new_package_name, new_object_name):

        from abjad import ABJCFG
        without_dirs = ['--without-dirs', 'build', '--without-dirs', '_build']

        directory = ABJCFG.ABJAD_ROOT_PATH

        print 'Updating codebase ...'
        print ''

        old_text = '{}.{}'.format(old_package_name, old_object_name)
        new_text = '{}.{}'.format(new_package_name, new_object_name)
        command = [directory, old_text, new_text, '--force', '--whole-words-only', '--verbose']
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)

        print ''

        if kind == 'function':
            old_text = 'test_{}_{}_'.format(old_package_name, old_object_name)
            new_text = 'test_{}_{}_'.format(new_package_name, new_object_name)
        elif kind == 'class':
            old_text = 'test_{}_'.format(old_object_name)
            new_text = 'test_{}_'.format(new_object_name)
        command = [directory, old_text, new_text, '--force', '--verbose']
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)

        print ''

        old_text = old_object_name
        new_text = new_object_name
        command = [directory, old_text, new_text, '--force', '--whole-words-only', '--verbose']
        command.extend(without_dirs)
        ReplaceInFilesScript()(command)

        print ''

    ### PUBLIC METHODS ###

    def process_args(self, args):
        iotools.clear_terminal()

        # print args

        kind = args.kind
        old_codebase, old_package_name, old_object_name = self._prompt_for_old_name(kind)
        new_codebase, new_package_name, new_object_name = self._prompt_for_new_name(kind)

        args = (kind, old_codebase, old_package_name, old_object_name,
            new_codebase, new_package_name, new_object_name)

        self._confirm_name_changes(*args)

        self._rename_old_module(*args)
        self._rename_old_test_files(*args)
        self._rename_old_api_page(*args)
        self._update_codebase(*args)

    def setup_argument_parser(self, parser):

        kind_group = parser.add_mutually_exclusive_group(required=True)

        kind_group.add_argument('-C', '--classes',
            action='store_const',
            const='class',
            dest='kind',
            help='rename classes',
            )

        kind_group.add_argument('-F', '--functions',
            action='store_const',
            const='function',
            dest='kind',
            help='rename functions',
            )

        #path_group = parser.add_mutually_exclusive_group(required=True)

        #path_group.add_argument('-X', '--experimental',
        #    action='store_const',
        #    const='experimental',
        #    dest='path',
        #    help='update Abjad abjad.tools directory',
        #    )

        #path_group.add_argument('-M', '--mainline',
        #    action='store_const',
        #    const='mainline',
        #    dest='path',
        #    help='update Abjad mainline directory',
        #    )

        parser.set_defaults(kind=False)



def get_abjad_version_string():
    '''.. versionadded:: 2.0

    Get Abjad version string::

        >>> configurationtools.get_abjad_version_string()
        '2.11'

    Return string.
    '''
    import abjad

    return abjad.__version__



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import measuretools
from abjad.tools import rhythmtreetools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class MetricalHierarchy(AbjadObject):
    '''.. versionadded:: 2.11

    A rhythm tree-based model of nested time signature groupings.

    The structure of the tree corresponds to the monotonically increasing 
    sequence of factors of the time signature's numerator.

    Each deeper level of the tree divides the previous by the next factor in sequence.

    Prime divisions greater than ``3`` are converted to sequences of ``2`` and ``3``
    summing to that prime. Hence ``5`` becomes ``3+2`` and ``7`` becomes ``3+2+2``.

    The metrical hierarchy models many parts of the common practice understanding of meter::

        >>> metrical_hierarchy = timesignaturetools.MetricalHierarchy((4, 4))

    ::

        >>> metrical_hierarchy
        MetricalHierarchy('(4/4 (1/4 1/4 1/4 1/4))')

    ::

        >>> print metrical_hierarchy.pretty_rtm_format
        (4/4 (
            1/4
            1/4
            1/4
            1/4))

    ::

        >>> print timesignaturetools.MetricalHierarchy((3, 4)).pretty_rtm_format
        (3/4 (
            1/4
            1/4
            1/4))

    ::

        >>> print timesignaturetools.MetricalHierarchy((6, 8)).pretty_rtm_format
        (6/8 (
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))))

    ::

        >>> print timesignaturetools.MetricalHierarchy((5, 4)).pretty_rtm_format
        (5/4 (
            (3/4 (
                1/4
                1/4
                1/4))
            (2/4 (
                1/4
                1/4))))

    ::

        >>> print timesignaturetools.MetricalHierarchy((5, 4),
        ...     decrease_durations_monotonically=False).pretty_rtm_format
        (5/4 (
            (2/4 (
                1/4
                1/4))
            (3/4 (
                1/4
                1/4
                1/4))))

    ::

        >>> print timesignaturetools.MetricalHierarchy((12, 8)).pretty_rtm_format
        (12/8 (
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))))

    Return metrical hierarchy object.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_decrease_durations_monotonically', '_denominator', '_numerator', '_root_node',)

    ### INITIALIZER ###

    def __init__(self, arg, decrease_durations_monotonically=True):

        def recurse(node, factors, denominator, decrease_durations_monotonically):
            if factors:
                factor, factors = factors[0], factors[1:]
                duration = node.duration / factor
                if factor in (2, 3, 4):
                    if factors:
                        for _ in range(factor):
                            child = rhythmtreetools.RhythmTreeContainer(duration=duration)
                            node.append(child)
                            recurse(child, factors, denominator, decrease_durations_monotonically)
                    else:
                        for _ in range(factor):
                            node.append(rhythmtreetools.RhythmTreeLeaf(duration=(1, denominator)))
                else:
                    parts = [3]
                    total = 3
                    while total < factor:
                        if decrease_durations_monotonically:
                            parts.append(2)
                        else:
                            parts.insert(0, 2)
                        total += 2
                    for part in parts:
                        grouping = rhythmtreetools.RhythmTreeContainer(duration=part * duration)
                        if factors:
                            for _ in range(part):
                                child = rhythmtreetools.RhythmTreeContainer(duration=duration)
                                grouping.append(child)
                                recurse(child, factors, denominator, decrease_durations_monotonically)
                        else:
                            for _ in range(part):
                                grouping.append(rhythmtreetools.RhythmTreeLeaf(duration=(1, denominator)))
                        node.append(grouping)
            else:
                node.extend([rhythmtreetools.RhythmTreeLeaf(duration=(1, denominator)) 
                    for _ in range(node.duration.numerator)])

        decrease_durations_monotonically = bool(decrease_durations_monotonically)

        if isinstance(arg, type(self)):
            root = arg.root_node
            numerator, denominator = arg.numerator, arg.denominator
            decrease_durations_monotonically = arg.decrease_durations_monotonically

        elif isinstance(arg, (str, rhythmtreetools.RhythmTreeContainer)):
            if isinstance(arg, str):
                parsed = rhythmtreetools.RhythmTreeParser()(arg)
                assert len(parsed) == 1
                root = parsed[0]
            else:
                root = arg
            for node in root.nodes:
                assert node.prolation == 1
            numerator, denominator = root.duration.numerator, root.duration.denominator

        elif isinstance(arg, (tuple, measuretools.Measure)) or \
            (hasattr(arg, 'numerator') and hasattr(arg, 'denominator')):
            if isinstance(arg, tuple):
                fraction = mathtools.NonreducedFraction(arg)
            elif isinstance(arg, measuretools.Measure):
                time_signature = contexttools.get_effective_time_signature(arg)
                fraction = mathtools.NonreducedFraction(
                    time_signature.numerator, time_signature.denominator)
            else:
                fraction = mathtools.NonreducedFraction(arg.numerator, arg.denominator)
            numerator, denominator = fraction.numerator, fraction.denominator
            factors = mathtools.factors(numerator)[1:]
            # group two nested levels of 2s into a 4
            if 1 < len(factors) and factors[0] == factors[1] == 2:
                factors[0:2] = [4]
            root = rhythmtreetools.RhythmTreeContainer(duration=fraction)
            recurse(root, factors, denominator, decrease_durations_monotonically)

        else:
            raise ValueError("Can't initialize {} from {!r}.".format(type(self).__name__, arg))

        self._root_node = root
        self._numerator = numerator
        self._denominator = denominator
        self._decrease_durations_monotonically = decrease_durations_monotonically

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other):
            if self.rtm_format == other.rtm_format:
                return True
        return False

    def __iter__(self):
        '''Iterate metrical hierarchy::

            >>> metrical_hierarchy = timesignaturetools.MetricalHierarchy((5, 4))

        ::

            >>> for x in metrical_hierarchy:
            ...    x
            ... 
            (NonreducedFraction(0, 4), NonreducedFraction(1, 4))
            (NonreducedFraction(1, 4), NonreducedFraction(2, 4))
            (NonreducedFraction(2, 4), NonreducedFraction(3, 4))
            (NonreducedFraction(0, 4), NonreducedFraction(3, 4))
            (NonreducedFraction(3, 4), NonreducedFraction(4, 4))
            (NonreducedFraction(4, 4), NonreducedFraction(5, 4))
            (NonreducedFraction(3, 4), NonreducedFraction(5, 4))
            (NonreducedFraction(0, 4), NonreducedFraction(5, 4))

        Yield pairs.
        '''
        def recurse(node):
            result = []
            for child in node:
                if isinstance(child, rhythmtreetools.RhythmTreeLeaf):
                    result.append(child)
                else:
                    result.extend(recurse(child))
            result.append(node)
            return result
        result = recurse(self.root_node)
        for x in result:
            start_offset = mathtools.NonreducedFraction(x.start_offset
                ).with_denominator(self.denominator)
            stop_offset = mathtools.NonreducedFraction(x.stop_offset
                ).with_denominator(self.denominator)
            yield start_offset, stop_offset

    def __repr__(self):
        return '{}({!r})'.format(self._class_name, self.rtm_format)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _keyword_argument_names(self):
        return ()

    @property
    def _positional_argument_values(self):
        return (self.rtm_format,)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def decrease_durations_monotonically(self):
        '''True if the metrical hierarchy divides large primes into collections of
        ``2`` and ``3`` that decrease monotonically.

        Example 1. Metrical hiearchy with durations that increase monotonically::

            >>> metrical_hierarchy = timesignaturetools.MetricalHierarchy((5, 4),
            ...     decrease_durations_monotonically=False)

        ::

            >>> metrical_hierarchy.decrease_durations_monotonically
            False

        ::

            >>> print metrical_hierarchy.pretty_rtm_format 
            (5/4 (
                (2/4 (
                    1/4
                    1/4))
                (3/4 (
                    1/4
                    1/4
                    1/4))))

        Example 2. Metrical hierarchy with durations that decrease monotonically::

            >>> metrical_hierarchy = timesignaturetools.MetricalHierarchy((5, 4),
            ...     decrease_durations_monotonically=True)

        ::

            >>> metrical_hierarchy.decrease_durations_monotonically
            True

        ::

            >>> print metrical_hierarchy.pretty_rtm_format 
            (5/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))))

        Return boolean.
        '''
        return self._decrease_durations_monotonically

    @property
    def denominator(self):
        r'''Beat hierarchy denominator::

            >>> metrical_hierarchy.denominator
            4

        Return positive integer.
        '''
        return self._denominator

    @property
    def depthwise_offset_inventory(self):
        '''Depthwise inventory of offsets at each grouping level::

            >>> for depth, offsets in enumerate(metrical_hierarchy.depthwise_offset_inventory):
            ...     print depth, offsets
            0 (Offset(0, 1), Offset(5, 4))
            1 (Offset(0, 1), Offset(3, 4), Offset(5, 4))
            2 (Offset(0, 1), Offset(1, 4), Offset(1, 2), Offset(3, 4), Offset(1, 1), Offset(5, 4))

        Return dictionary.
        '''
        inventory = []
        for depth, nodes in sorted(self.root_node.depthwise_inventory.items()):
            offsets = []
            for node in nodes:
                offsets.append(durationtools.Offset(node.start_offset))
            offsets.append(durationtools.Offset(self.numerator, self.denominator))
            inventory.append(tuple(offsets))
        return tuple(inventory)

    @property
    def duration(self):
        '''Beat hierarchy duration::

            >>> metrical_hierarchy.duration
            Duration(5, 4)

        Return duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def graphviz_format(self):
        '''Graphviz format of hierarchy's root node:

        ::

            >>> print metrical_hierarchy.graphviz_format
            digraph G {
                node_0 [label="5/4",
                    shape=triangle];
                node_1 [label="3/4",
                    shape=triangle];
                node_2 [label="1/4",
                    shape=box];
                node_3 [label="1/4",
                    shape=box];
                node_4 [label="1/4",
                    shape=box];
                node_5 [label="2/4",
                    shape=triangle];
                node_6 [label="1/4",
                    shape=box];
                node_7 [label="1/4",
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_5;
                node_1 -> node_2;
                node_1 -> node_3;
                node_1 -> node_4;
                node_5 -> node_6;
                node_5 -> node_7;
            }

        ::

            >>> iotools.graph(metrical_hierarchy) # doctest: +SKIP

        Return string.
        '''
        return self.root_node.graphviz_format

    @property
    def implied_time_signature(self):
        '''Implied time signature:

        ::

            >>> timesignaturetools.MetricalHierarchy((4, 4)).implied_time_signature
            TimeSignatureMark((4, 4))

        Return TimeSignatureMark object.
        '''
        return contexttools.TimeSignatureMark(self.root_node.duration)

    @property
    def numerator(self):
        r'''Beat hierarchy numerator::

            >>> metrical_hierarchy.numerator
            5

        Return positive integer.
        '''
        return self._numerator

    @property
    def pretty_rtm_format(self):
        '''Beat hiearchy pretty RTM format::

            >>> print metrical_hierarchy.pretty_rtm_format
            (5/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))))

        Return string.
        '''
        return self.root_node.pretty_rtm_format

    @property
    def root_node(self):
        '''Beat hiearchy root node::

            >>> metrical_hierarchy.root_node
            RhythmTreeContainer(
                children=(
                    RhythmTreeContainer(
                        children=(
                            RhythmTreeLeaf(
                                duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                duration=Duration(1, 4),
                                is_pitched=True
                                )
                            ),
                        duration=NonreducedFraction(3, 4)
                        ),
                    RhythmTreeContainer(
                        children=(
                            RhythmTreeLeaf(
                                duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                duration=Duration(1, 4),
                                is_pitched=True
                                )
                            ),
                        duration=NonreducedFraction(2, 4)
                        )
                    ),
                duration=NonreducedFraction(5, 4)
                )

        Return rhythm tree node.
        '''
        return self._root_node

    @property
    def rtm_format(self):
        '''Beat hierarchy RTM format::

            >>> metrical_hierarchy.rtm_format
            '(5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4))))'

        Return string.
        '''
        return self._root_node.rtm_format

    @property
    def storage_format(self):
        '''Beat hierarchy storage format::

            >>> print metrical_hierarchy.storage_format
            timesignaturetools.MetricalHierarchy(
                '(5/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4))))'
                )

        Return string.
        '''
        return AbjadObject.storage_format.fget(self)

    ### PRIVATE METHODS ###

    def _get_recurser(self):


        return recurse

    ### PUBLIC METHODS ###

    def generate_offset_kernel_to_denominator(self, denominator, normalize=True):
        r'''Generate a dictionary of all offsets in a metrical hierarchy up
        to `denominator`, where the keys are the offsets and the values
        are the normalized weights of those offsets:
        
        ::
        
            >>> metrical_hierarchy = timesignaturetools.MetricalHierarchy((4, 4))
            >>> kernel = metrical_hierarchy.generate_offset_kernel_to_denominator(8)
            >>> for offset, weight in sorted(kernel.kernel.iteritems()):
            ...     print '{}\t{}'.format(offset, weight)
            ...
            0       3/16
            1/8     1/16
            1/4     1/8
            3/8     1/16
            1/2     1/8
            5/8     1/16
            3/4     1/8
            7/8     1/16
            1       3/16

        This is useful for testing how strongly a collection of offsets
        responds to a given metrical hierarchy.
        
        Return dictionary.
        '''
        from abjad.tools import timesignaturetools
        assert mathtools.is_positive_integer_power_of_two(
            denominator / self.denominator)

        inventory = list(self.depthwise_offset_inventory)
        old_flag_count = durationtools.Duration(1, self.denominator).flag_count
        new_flag_count = durationtools.Duration(1, denominator).flag_count
        extra_depth = new_flag_count - old_flag_count
        for _ in range(extra_depth):
            old_offsets = inventory[-1]
            new_offsets = []
            for first, second in sequencetools.iterate_sequence_pairwise_strict(old_offsets):
                new_offsets.append(first)
                new_offsets.append((first + second) / 2)
            new_offsets.append(old_offsets[-1])
            inventory.append(tuple(new_offsets))

        total = 0
        kernel = {}
        for offsets in inventory:
            for offset in offsets:
                if offset not in kernel:
                    kernel[offset] = 0
                kernel[offset] += 1
                total += 1
        
        if normalize:    
            for offset, response in kernel.iteritems():
                kernel[offset] = durationtools.Multiplier(response, total)
        
        return timesignaturetools.MetricalKernel(kernel)



def get_articulation_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Get exactly one articulation attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> marktools.Articulation('staccato')(staff[0])
        Articulation('staccato')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 -\staccato
            d'8
            e'8
            f'8
        }

    ::

        >>> marktools.get_articulation_attached_to_component(staff[0])
        Articulation('staccato')(c'8)

    Return one articulation.

    Raise missing mark error when no articulation is attached.

    Raise extra mark error when more than one articulation is attached.
    '''
    from abjad.tools import marktools

    articulations = marktools.get_articulations_attached_to_component(component)
    if not articulations:
        raise MissingMarkError
    elif 1 < len(articulations):
        raise ExtraMarkError
    else:
        return articulations[0]



import os
from abjad import *


def make_nested_tuplet(tuplet_duration, outer_tuplet_proportions, inner_tuplet_subdivision_count):
    outer_tuplet = tuplettools.make_tuplet_from_duration_and_ratio(
        tuplet_duration, outer_tuplet_proportions)
    inner_tuplet_proportions = inner_tuplet_subdivision_count * [1]
    right_tie_chain = tietools.get_tie_chain(outer_tuplet.leaves[-2])
    tietools.tie_chain_to_tuplet_with_ratio(right_tie_chain, inner_tuplet_proportions)
    return outer_tuplet

def make_row_of_nested_tuplets(tuplet_duration, outer_tuplet_proportions, column_count):
    row_of_nested_tuplets = []
    for n in range(column_count):
        inner_tuplet_subdivision_count = n + 1
        nested_tuplet = make_nested_tuplet(
            tuplet_duration, outer_tuplet_proportions, inner_tuplet_subdivision_count)
        row_of_nested_tuplets.append(nested_tuplet)
    return row_of_nested_tuplets

def make_rows_of_nested_tuplets(tuplet_duration, row_count, column_count):
    rows_of_nested_tuplets = []
    for n in range(row_count):
        outer_tuplet_proportions = (1, n + 1, 1)
        row_of_nested_tuplets = make_row_of_nested_tuplets(
            tuplet_duration, outer_tuplet_proportions, column_count)    
        rows_of_nested_tuplets.append(row_of_nested_tuplets)
    return rows_of_nested_tuplets

def make_lilypond_file(tuplet_duration, row_count, column_count):
    rows_of_nested_tuplets = make_rows_of_nested_tuplets(tuplet_duration, row_count, column_count)
    all_nested_tuplets = sequencetools.flatten_sequence(rows_of_nested_tuplets)
    staff = stafftools.RhythmicStaff(all_nested_tuplets)
    time_signature  = contexttools.TimeSignatureMark((1, 4))
    time_signature.attach(staff) 
    score = Score([staff])
    configure_score(score)
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    configure_lilypond_file(lilypond_file)
    return lilypond_file

def configure_score(score):
    score.set.proportional_notation_duration = schemetools.SchemeMoment(1, 56)
    score.set.tuplet_full_length = True
    score.override.bar_line.stencil = False
    score.override.bar_number.transparent = True
    score.override.spacing_spanner.uniform_stretching = True
    score.override.spacing_spanner.strict_note_spacing = True
    score.override.time_signature.stencil = False
    score.override.tuplet_bracket.padding = 2
    score.override.tuplet_bracket.staff_padding = 4
    score.override.tuplet_number.text = schemetools.Scheme('tuplet-number::calc-fraction-text')
    pass

def configure_lilypond_file(lilypond_file):
    lilypond_file.default_paper_size = '11x17', 'portrait'
    lilypond_file.global_staff_size = 12
    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    lilypond_file.paper_block.ragged_bottom = True
    lilypond_file.paper_block.system_system_spacing = layouttools.make_spacing_vector(0, 0, 8, 0)
    pass

if __name__ == '__main__':
    os.system('clear')
    lilypond_file = make_lilypond_file(Duration(1, 4), 10, 6)
    show(lilypond_file)



from abjad.tools.contexttools.TempoMark import TempoMark
from abjad.tools.datastructuretools.ObjectInventory import ObjectInventory


class TempoMarkInventory(ObjectInventory):
    r'''.. versionadded:: 2.7

    Abjad model of an ordered list of tempo marks::

        >>> inventory = contexttools.TempoMarkInventory([
        ...     ('Andante', Duration(1, 8), 72), 
        ...     ('Allegro', Duration(1, 8), 84)])

    ::

        >>> for tempo_mark in inventory:
        ...     tempo_mark
        ...
        TempoMark('Andante', Duration(1, 8), 72)
        TempoMark('Allegro', Duration(1, 8), 84)

    Tempo mark inventories implement list interface and are mutable.
    '''

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        return TempoMark



from abjad import *


def test_labeltools_label_leaves_in_expr_with_pitch_numbers_01():
    '''Works on notes, rests and chords.
    '''

    leaves = leaftools.make_leaves([None, 12, (13, 14, 15), None], [(1, 4)])
    t = Staff(leaves)
    labeltools.label_leaves_in_expr_with_pitch_numbers(t)

    r'''
    \new Staff {
        r4
        c''4
            _ \markup {
                \small
                    12
                }
        <cs'' d'' ef''>4
            _ \markup {
                \column
                    {
                        \small
                            15
                        \small
                            14
                        \small
                            13
                    }
                }
        r4
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tr4\n\tc''4\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t12\n\t\t\t}\n\t<cs'' d'' ef''>4\n\t\t_ \\markup {\n\t\t\t\\column\n\t\t\t\t{\n\t\t\t\t\t\\small\n\t\t\t\t\t\t15\n\t\t\t\t\t\\small\n\t\t\t\t\t\t14\n\t\t\t\t\t\\small\n\t\t\t\t\t\t13\n\t\t\t\t}\n\t\t\t}\n\tr4\n}"



from abjad import *
import py.test


def test_Spanner___init___01():
    '''Init empty spanner.'''

    p = beamtools.BeamSpanner()
    assert len(p) == 0
    assert p[:] == []


def test_Spanner___init___02():
    '''Init nonempty spanner.'''

    t = Container("c'8 d'8 e'8 f'8")
    p = beamtools.BeamSpanner(t[:])

    r'''
    {
        c'8 [
        d'8
        e'8
        f'8 ]
    }
    '''

    assert len(p) == 4
    assert p[:] == t[:]


def test_Spanner___init___03():
    '''Spanner is abstract.'''

    assert py.test.raises(TypeError, 'p = spannertools.Spanner()')



from abjad import *
import py.test


def test_RhythmTreeContainer_index_01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(
        duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(duration=1)
    container = rhythmtreetools.RhythmTreeContainer(
        duration=1, children=[leaf_a, subcontainer, leaf_d])

    assert container.index(leaf_a) == 0
    assert container.index(subcontainer) == 1
    assert container.index(leaf_d) == 2

    py.test.raises(ValueError, 'container.index(leaf_b)')
    py.test.raises(ValueError, 'container.index(leaf_c)')



import os


def read_test_output(full_file_name, current_function_name):
    r'''Read test output.
    '''
    segment_ly_file_name = '{}.ly'.format(current_function_name)
    directory_name = os.path.dirname(full_file_name)
    segment_ly_path_name = os.path.join(directory_name, segment_ly_file_name)
    return file(segment_ly_path_name, 'r').read()



from abjad import *


def test_componenttools_get_lineage_of_component_that_stop_with_component_01( ):

    t = Voice(Container(notetools.make_repeated_notes(2)) * 3)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Voice {
        {
            c'8
            d'8
        }
        {
            e'8
            f'8
        }
        {
            g'8
            a'8
        }
    }
    '''

    result = componenttools.get_lineage_of_component_that_stop_with_component(t)

    "[Voice{3}, Container(g'8, a'8), Note(a', 8)]"

    assert len(result) == 3
    assert t in result
    assert t[-1] in result
    assert t[-1][-1] in result

    result = componenttools.get_lineage_of_component_that_stop_with_component(t[-1])

    "[Container(g'8, a'8), Note(a', 8), Voice{3}]"

    assert len(result) == 3
    assert t in result
    assert t[-1] in result
    assert t[-1][-1] in result

    result = componenttools.get_lineage_of_component_that_stop_with_component(t[-1][-1])

    "[Note(a', 8), Container(g'8, a'8), Voice{3}]"

    assert len(result) == 3
    assert t in result
    assert t[-1] in result
    assert t[-1][-1] in result



from abjad.tools import durationtools


def shift_aggregate_offset_to_rational(intervals, rational):
    '''Shift the aggregate offset of `intervals` to `rational` ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(-1, 3)
        >>> b = TimeInterval(6, 12)
        >>> c = TimeInterval(9, 16)
        >>> tree = TimeIntervalTree([a, b, c])
        >>> timeintervaltools.shift_aggregate_offset_to_rational(tree, Fraction(10, 7))
        TimeIntervalTree([
            TimeInterval(Offset(10, 7), Offset(38, 7), {}),
            TimeInterval(Offset(59, 7), Offset(101, 7), {}),
            TimeInterval(Offset(80, 7), Offset(129, 7), {})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    rational = durationtools.Duration(rational)

    if not tree or rational == tree.start:
        return tree

    return timeintervaltools.TimeIntervalTree([
        x.shift_by_rational(rational - tree.start) for x in tree
    ])



from abjad.tools.quantizationtools.QEvent import QEvent


class SilentQEvent(QEvent):
    '''A ``QEvent`` which indicates the onset of a period of silence
    in a ``QEventSequence``:

        >>> q_event = quantizationtools.SilentQEvent(1000)
        >>> q_event
        quantizationtools.SilentQEvent(
            durationtools.Offset(1000, 1),
            attachments=()
            )

    Return ``SilentQEvent`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_attachments', '_offset',)

    ### INITIALIZER ###

    def __init__(self, offset, attachments=None, index=None):
        QEvent.__init__(self, offset, index=index)
        if attachments is None:
            attachments = ()
        else:
            attachments = tuple(attachments)
        self._attachments = attachments

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other) and \
            self._offset == other._offset and \
            self._attachments == other._attachments and \
            self._index == other._index:
            return True
        return False

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def attachments(self):
        return self._attachments



from abjad.tools import abctools
from abjad.tools import documentationtools
import os


class CodeBlock(abctools.AbjadObject):

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_ending_line_number', '_hide', '_lines', '_processed_results',
        '_starting_line_number', '_strip_prompt')

    ### INITIALIZER ###

    def __init__(self, lines, starting_line_number, ending_line_number,
        hide=False, strip_prompt=False):
        assert starting_line_number <= ending_line_number
        self._lines = tuple(lines)
        self._starting_line_number = starting_line_number
        self._ending_line_number = ending_line_number
        self._hide = bool(hide)
        self._strip_prompt = bool(strip_prompt)
        self._processed_results = None

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other) and \
            self.lines == other.lines and \
            self.starting_line_number == other.starting_line_number and \
            self.ending_line_number == other.ending_line_number and \
            self.hide == other.hide and \
            self.strip_prompt == other.strip_prompt:
            return True
        return False

    ### SPECIAL METHODS ###

    def __call__(self, processor, pipe, image_count=0, directory=None,
        image_prefix='image', verbose=False):

        assert isinstance(pipe, documentationtools.Pipe)

        if verbose:
            print '\nCODEBLOCK: {}:{}'.format(
                self.starting_line_number, 
                self.ending_line_number
                )

        grouped_results = []
        result = []

        pipe.write('\n')

        previous_line_was_empty = False

        for line in self.lines:

            processor.update_status(line)

            hide = self.hide

            if not self.strip_prompt or not previous_line_was_empty:
                current = self.read(pipe)

            if line.endswith('<hide'):
                hide = True
                line = line.rpartition('<hide')[0]

            page_range = None
            if '<page' in line:
                parts = line.rpartition('<page')
                line = parts[0]
                page_range = parts[-1].strip()
                if '-' in page_range:
                    parts = page_range.partition('-')
                    page_range = range(int(parts[0]), int(parts[-1]) + 1)
                else:
                    page_range = [int(page_range)]

            if not hide:
                current[-1] += line
                result.extend(current)

            if verbose:
                for x in current:
                    print x

            if line.startswith('show('):

                image_count += 1
                file_name = '{}-{}'.format(image_prefix, image_count)
                docs = False

                if ',' in line:
                    object_name = line.split(',')[0][5:].strip()
                    if 'docs=True' in line:
                        docs = True
                else:
                    object_name = line.partition(')')[0][5:]

                if directory:
                    command = "iotools.write_expr_to_ly({}, {!r}, docs={})".format(
                        object_name, os.path.join(directory, file_name), docs)
                else:
                    command = "iotools.write_expr_to_ly({}, {!r}, docs={})".format(
                        object_name, file_name, docs)

                pipe.write(command)
                grouped_results.append(result)

                image_dict = {
                    'file_name': file_name,
                    'image_count': image_count,
                    'image_prefix': image_prefix,
                    'page_range': page_range,
                }
                grouped_results.append(image_dict)

                result = []
                pipe.write('\n')
                previous_line_was_empty = False

            else:
                if not self.strip_prompt or len(line.strip()):
                    pipe.write(line)
                    pipe.write('\n')
                    previous_line_was_empty = False
                else:
                    previous_line_was_empty = True
                #print '  LINE?', line
                #if len(line.strip()):
                #    #print '  PIPE:', line
                #    pipe.write(line)
                #    pipe.write('\n')
                #    previous_line_was_empty = False
                #else:
                #    previous_line_was_empty = True

        result.extend(self.read(pipe))
        if result[-1] == '>>> ':
            result.pop()

        pipe.write('\n')
        result.extend(self.read(pipe))
        if result[-1] == '>>> ':
            result.pop()

        grouped_results.append(result)

        if self.strip_prompt:
            for result in [group for group in grouped_results if isinstance(group, list)]:
                for i, line in enumerate(result):
                    if line.startswith(('>>> ', '... ')):
                        result[i] = line[4:]
                while not result[-1]:
                    result.pop()

        for i, result in enumerate(grouped_results):
            if isinstance(result, list):
                grouped_results[i] = tuple(result)

        self._processed_results = tuple([x for x in grouped_results if x])

        return image_count

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def ending_line_number(self):
        return self._ending_line_number

    @property
    def hide(self):
        return self._hide

    @property
    def lines(self):
        return self._lines

    @property
    def processed_results(self):
        return self._processed_results

    @property
    def starting_line_number(self):
        return self._starting_line_number

    @property
    def strip_prompt(self):
        return self._strip_prompt
        
    ### PUBLIC METHODS ###

    def read(self, pipe):
        # Guarantee we make it to the next prompt.
        # Exceptions sometimes take longer than expected.
        result = pipe.read_wait().replace('\t', '    ').split('\n')
        if result[-1] == '':
            result.pop()
        while result[-1] not in ('>>> ', '... '):
            new = pipe.read_wait().replace('\t', '    ').split('\n')
            if new[-1] == '':
                new.pop()
            result.extend(new)
        return result



from abjad import *


def test_Tuplet_timespan_01():

    staff = Staff(r"c'4 d'4 \times 2/3 { e'4 f'4 g'4 }")

    r'''
    \new Staff {
        c'4
        d'4
        \times 2/3 {
            e'4
            f'4
            g'4
        }
    }
    '''

    assert staff.timespan == timespantools.Timespan(0, 1)
    assert staff[0].timespan == timespantools.Timespan(0, (1, 4))
    assert staff[1].timespan == timespantools.Timespan((1, 4), (1, 2))
    assert staff[-1].timespan == timespantools.Timespan((1, 2), 1)



def get_grace_containers_attached_to_leaf(leaf, kind=None):
    r'''.. versionadded:: 2.0

    Example 1. Get all grace containers attached to `leaf`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> gracetools.GraceContainer([Note("cs'16")], kind='grace')(staff[1])
        Note("d'8")
        >>> gracetools.GraceContainer([Note("ds'16")], kind='after')(staff[1])
        Note("d'8")

    ::

        >>> f(staff)
        \new Staff {
            c'8
            \grace {
                cs'16
            }
            \afterGrace
            d'8
            {
                ds'16
            }
            e'8
            f'8
        }

    ::

        >>> gracetools.get_grace_containers_attached_to_leaf(staff[1])
        (GraceContainer(cs'16), GraceContainer(ds'16))

    Example 2. Get only (proper) grace containers attached to `leaf`::

        >>> gracetools.get_grace_containers_attached_to_leaf(staff[1], kind='grace')
        (GraceContainer(cs'16),)

    Example 3. Get only after grace containers attached to `leaf`::

        >>> gracetools.get_grace_containers_attached_to_leaf(staff[1], kind='after')
        (GraceContainer(ds'16),)

    Return tuple.
    '''

    result = []
    if kind in (None, 'grace') and hasattr(leaf, '_grace'):
        result.append(leaf._grace)
    if kind in (None, 'after') and hasattr(leaf, '_after_grace'):
        result.append(leaf._after_grace)
    return tuple(result)



from abjad import *


def test_labeltools_label_leaves_in_expr_with_written_leaf_duration_01():

    t = tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    labeltools.label_leaves_in_expr_with_written_leaf_duration(t)

    r'''
    \times 2/3 {
        c'8
            _ \markup {
                \small
                    1/8
                }
        d'8
            _ \markup {
                \small
                    1/8
                }
        e'8
            _ \markup {
                \small
                    1/8
                }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\times 2/3 {\n\tc'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t1/8\n\t\t\t}\n\td'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t1/8\n\t\t\t}\n\te'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t1/8\n\t\t\t}\n}"



from abjad import *


def test_leaftools_fuse_leaves_in_container_once_by_counts_01():

    t = Voice(notetools.make_repeated_notes(5, Duration(1, 16)))
    spannertools.SlurSpanner(t[:])
    leaftools.fuse_leaves_in_container_once_by_counts(t, [1, 2, 2], klass=Note, decrease_durations_monotonically=True)

    r'''
    \new Voice {
      c'16 (
      c'8
      c'8 )
    }
    '''

    assert t.lilypond_format == "\\new Voice {\n\tc'16 (\n\tc'8\n\tc'8 )\n}"


def test_leaftools_fuse_leaves_in_container_once_by_counts_02():

    t = Voice(notetools.make_repeated_notes(5))
    spannertools.SlurSpanner(t[:])
    leaftools.fuse_leaves_in_container_once_by_counts(t, [5], klass=Note, decrease_durations_monotonically=True)

    r'''
    \new Voice {
      c'2 ( ~
      c'8 )
    }
    '''

    assert t.lilypond_format == "\\new Voice {\n\tc'2 ( ~\n\tc'8 )\n}"


def test_leaftools_fuse_leaves_in_container_once_by_counts_03():

    t = Voice(notetools.make_repeated_notes(5))
    leaftools.fuse_leaves_in_container_once_by_counts(t, [1, 2, 2], klass=Rest, decrease_durations_monotonically=True)

    r'''
    \new Voice {
      r8
      r4
      r4
    }
    '''

    assert t.lilypond_format == '\\new Voice {\n\tr8\n\tr4\n\tr4\n}'


def test_leaftools_fuse_leaves_in_container_once_by_counts_04():

    t = Voice(notetools.make_repeated_notes(5))
    spannertools.SlurSpanner(t)
    leaftools.fuse_leaves_in_container_once_by_counts(t, [5], klass=Note, decrease_durations_monotonically=False)

    r'''
    \new Voice {
      c'8 ( ~
      c'2 )
    }
    '''

    assert t.lilypond_format == "\\new Voice {\n\tc'8 ( ~\n\tc'2 )\n}"



from abjad import *


def test_TieChain_all_leaves_are_in_same_parent_01():


    staff = notetools.make_repeated_notes(4)
    tietools.TieSpanner(staff[:])

    assert tietools.get_tie_chain(staff[0]).all_leaves_are_in_same_parent


def test_TieChain_all_leaves_are_in_same_parent_02():

    staff = Staff(2 * Measure((2, 8), "c'8 c'8"))
    tietools.TieSpanner(staff.leaves[1:3])

    r'''
    \new Staff {
            \time 2/8
            c'8
            c'8 ~
            \time 2/8
            c'8
            c'8
    }
    '''

    assert tietools.get_tie_chain(staff.leaves[0]).all_leaves_are_in_same_parent
    assert not tietools.get_tie_chain(staff.leaves[1]).all_leaves_are_in_same_parent
    assert not tietools.get_tie_chain(staff.leaves[2]).all_leaves_are_in_same_parent
    assert tietools.get_tie_chain(staff.leaves[3]).all_leaves_are_in_same_parent


def test_TieChain_all_leaves_are_in_same_parent_03():

    staff = Staff(r"\times 2/3 { c'8 c'8 c'8 ~ } \times 2/3 { c'8 c'8 c'8 }")

    r'''
    \new Staff {
        \times 2/3 {
            c'8
            c'8
            c'8 ~
        }
        \times 2/3 {
            c'8
            c'8
            c'8
        }
    }
    '''

    assert tietools.get_tie_chain(staff.leaves[0]).all_leaves_are_in_same_parent
    assert tietools.get_tie_chain(staff.leaves[1]).all_leaves_are_in_same_parent
    assert not tietools.get_tie_chain(staff.leaves[2]).all_leaves_are_in_same_parent
    assert not tietools.get_tie_chain(staff.leaves[3]).all_leaves_are_in_same_parent
    assert tietools.get_tie_chain(staff.leaves[4]).all_leaves_are_in_same_parent
    assert tietools.get_tie_chain(staff.leaves[5]).all_leaves_are_in_same_parent



from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.rhythmmakertools.BurnishedRhythmMaker import BurnishedRhythmMaker


class OutputBurnishedTaleaRhythmMaker(BurnishedRhythmMaker):
    r'''.. versionadded:: 2.8

    Output-burnished talea rhythm-maker.

    Configure the rhythm-maker at initialization::

        >>> talea, talea_denominator, prolation_addenda = [1, 2, 3], 16, [0, 2]
        >>> lefts, middles, rights = [-1], [0], [-1]
        >>> left_lengths, right_lengths = [1], [1]
        >>> secondary_divisions = [9]
        >>> maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        ... talea, talea_denominator, prolation_addenda, lefts, middles, rights, 
        ... left_lengths, right_lengths, secondary_divisions)

    Then call the rhythm-maker on arbitrary divisions::

        >>> divisions = [(3, 8), (4, 8)]
        >>> music = maker(divisions)

    The resulting Abjad objects can be included in any score::

        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 3/8
                {
                    r16
                    c'8
                    c'8.
                }
            }
            {
                \time 4/8
                \fraction \times 3/5 {
                    c'16
                    c'8
                    c'8
                }
                {
                    c'16
                    c'16
                    c'8
                    r16
                }
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### PRIVATE METHODS ###

    def _burnish_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths = quintuplet
        burnished_divisions = []
        left_length = left_lengths[0]
        left = lefts[:left_length]
        right_length = right_lengths[0]
        right = rights[:right_length]
        if len(divisions) == 1:
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            available_right_length = len(divisions[0]) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[0]) - left_length - right_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[0], [left_length, middle_length, right_length], cyclic=False, overhang=False)
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        else:
            ## first division
            available_left_length = len(divisions[0])
            left_length = min([left_length, available_left_length])
            middle_length = len(divisions[0]) - left_length
            left = left[:left_length]
            middle = middle_length * [middles[0]]
            left_part, middle_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[0], [left_length, middle_length], cyclic=False, overhang=False)
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            burnished_division = left_part + middle_part
            burnished_divisions.append(burnished_division)
            ## middle divisions
            for division in divisions[1:-1]:
                middle_part = division
                middle = len(division) * [middles[0]]
                middle_part = self._burnish_division_part(middle_part, middle)
                burnished_division = middle_part
                burnished_divisions.append(burnished_division)
            ## last division:
            available_right_length = len(divisions[-1])
            right_length = min([right_length, available_right_length])
            middle_length = len(divisions[-1]) - right_length
            right = right[:right_length]
            middle = middle_length * [middles[0]]
            middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                divisions[-1], [middle_length, right_length], cyclic=False, overhang=False)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions



from abjad.tools import componenttools
from abjad.tools import spannertools


def is_component_with_beam_spanner_attached(expr):
    '''.. versionadded:: 2.0

    True when `expr` is component with beam spanner attached::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)

    ::

        >>> beamtools.is_component_with_beam_spanner_attached(staff[0])
        True

    Otherwise false::

        >>> note = Note("c'8")

    ::

        >>> beamtools.is_component_with_beam_spanner_attached(note)
        False

    Return boolean.

    .. versionchanged:: 2.9
        renamed ``spannertools.is_component_with_beam_spanner_attached()`` to
        ``beamtools.is_component_with_beam_spanner_attached()``.
    '''
    from abjad.tools import beamtools

    if not isinstance(expr, componenttools.Component):
        return False

    return bool(spannertools.get_spanners_attached_to_component(expr, beamtools.BeamSpanner))



from abjad import *


def test_pitchtools_chromatic_pitch_number_to_chromatic_pitch_triple_01():

    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(12, 'mixed')
    assert t == ('c', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(13, 'mixed')
    assert t == ('c', pitchtools.Accidental('s'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(14, 'mixed')
    assert t == ('d', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(15, 'mixed')
    assert t == ('e', pitchtools.Accidental('f'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(16, 'mixed')
    assert t == ('e', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(17, 'mixed')
    assert t == ('f', pitchtools.Accidental(''), 5)


def test_pitchtools_chromatic_pitch_number_to_chromatic_pitch_triple_02():

    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(12, 'sharps')
    assert t == ('c', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(13, 'sharps')
    assert t == ('c', pitchtools.Accidental('s'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(14, 'sharps')
    assert t == ('d', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(15, 'sharps')
    assert t == ('d', pitchtools.Accidental('s'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(16, 'sharps')
    assert t == ('e', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(17, 'sharps')
    assert t == ('f', pitchtools.Accidental(''), 5)


def test_pitchtools_chromatic_pitch_number_to_chromatic_pitch_triple_03():

    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(12, 'flats')
    assert t == ('c', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(13, 'flats')
    assert t == ('d', pitchtools.Accidental('f'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(14, 'flats')
    assert t == ('d', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(15, 'flats')
    assert t == ('e', pitchtools.Accidental('f'), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(16, 'flats')
    assert t == ('e', pitchtools.Accidental(''), 5)
    t = pitchtools.chromatic_pitch_number_to_chromatic_pitch_triple(17, 'flats')
    assert t == ('f', pitchtools.Accidental(''), 5)



from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__marks__Markup_01():
    target = Staff([Note(0, 1)])
    markuptools.Markup('hello!', Up)(target[0])

    r'''\new Staff {
        c'1 ^ \markup { hello! }
    }
    '''

    input = r'''\new Staff { c'1 ^ "hello!" }'''

    parser = LilyPondParser()
    result = parser(input)
    assert target.lilypond_format == result.lilypond_format and target is not result
    assert 1 == len(markuptools.get_markup_attached_to_component(result[0]))


def test_LilyPondParser__marks__Markup_02():

    target = Staff([Note(0, (1, 4))])
    markuptools.Markup(['X', 'Y', 'Z', 'a b c'], Down)(target[0])

    r'''\new Staff {
        c'4
            _ \markup {
                X
                Y
                Z
                "a b c"
                }
    }
    '''

    input = r'''\new Staff { c' _ \markup { X Y Z "a b c" } }'''

    parser = LilyPondParser()
    result = parser(input)
    assert target.lilypond_format == result.lilypond_format and target is not result
    assert 1 == len(markuptools.get_markup_attached_to_component(result[0]))


def test_LilyPondParser__marks__Markup_03():
    '''Articulations following markup block are (re)lexed correctly after
    returning to the "notes" lexical state after popping the "markup lexical state.
    '''

    target = Staff([Note(0, (1, 4)), Note(2, (1, 4))])
    markuptools.Markup('hello', Up)(target[0])
    marktools.Articulation('.')(target[0])

    r'''\new Staff {
        c'4 -\staccato ^ \markup { hello }
        d'4
    }
    '''

    input = r'''\new Staff { c' ^ \markup { hello } -. d' }'''

    parser = LilyPondParser()
    result = parser(input)
    assert target.lilypond_format == result.lilypond_format and target is not result
    assert 1 == len(markuptools.get_markup_attached_to_component(result[0]))


def test_LilyPondParser__marks__Markup_04():
    command1 = markuptools.MarkupCommand('bold', ['A', 'B', 'C'])
    command2 = markuptools.MarkupCommand('italic', '123')
    markup = markuptools.Markup((command1, command2))

    parser = LilyPondParser()
    result = parser(markup.lilypond_format)

    assert isinstance(result, markuptools.Markup)
    assert result.lilypond_format == markup.lilypond_format


def test_LilyPondParser__marks__Markup_05():
    command = r'\markup { \char ##x03EE }'
    parser = LilyPondParser()
    result = parser(command)
    assert result.lilypond_format == '\\markup { \\char #1006 }'



from abjad import *


def test_labeltools_color_leaf_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    beamtools.BeamSpanner(staff.leaves)

    r'''
    \new Staff {
        c'8 [
        d'8
        e'8
        f'8 ]
    }
    '''

    labeltools.color_leaf(staff[0], 'red')

    r'''
    \new Staff {
        \once \override Accidental #'color = #red
        \once \override Dots #'color = #red
        \once \override NoteHead #'color = #red
        c'8 [
        d'8
        e'8
        f'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\t\\once \\override Accidental #'color = #red\n\t\\once \\override Dots #'color = #red\n\t\\once \\override NoteHead #'color = #red\n\tc'8 [\n\td'8\n\te'8\n\tf'8 ]\n}"



'''Abjad ``labeltools`` package.

Dependencies:

The ``labeltools`` package is a high-level labeling package.
The package may import essentially all of the core packages
at top level.
'''
from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'




def diatonic_pitch_name_to_chromatic_pitch_number(diatonic_pitch_name):
    '''.. versionadded:: 2.0

    Change `diatonic_pitch_name` to chromatic pitch number::

        >>> pitchtools.diatonic_pitch_name_to_chromatic_pitch_number("c''")
        12

    Return integer.
    '''
    from abjad.tools import pitchtools

    diatonic_pitch_number = pitchtools.diatonic_pitch_name_to_diatonic_pitch_number(diatonic_pitch_name)

    return pitchtools.diatonic_pitch_number_to_chromatic_pitch_number(diatonic_pitch_number)



import copy
from abjad.tools.leaftools.Leaf import Leaf


class Rest(Leaf):
    '''Abjad model of a rest:

    ::

        >>> rest = Rest((3, 16))

    ::

        >>> rest
        Rest('r8.')

    ::

        >>> show(rest) # doctest: +SKIP

    Return Rest instance.
    '''

    ### CLASS ATTRIBUTES ###

    # TODO: add vertical positioning pitch only as needed #
    __slots__ = ('_vertical_positioning_pitch', )

    _default_positional_input_arguments = (repr('r4'), )

    ### INITIALIZER ###

    # TODO: use LilyPond parser for initialization
    def __init__(self, *args, **kwargs):
        from abjad.tools import lilypondparsertools

        if len(args) == 1 and isinstance(args[0], str):
            input = '{{ {} }}'.format(args[0])
            parsed = lilypondparsertools.LilyPondParser()(input)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            args = [parsed[0]]
            #written_duration = args[0].strip('r')
            #lilypond_multiplier = None

        if len(args) == 1 and isinstance(args[0], Leaf):
            leaf = args[0]
            written_duration = leaf.written_duration
            lilypond_multiplier = leaf.duration_multiplier
            self._copy_override_and_set_from_leaf(leaf)
        elif len(args) == 1 and not isinstance(args[0], str):
            written_duration = args[0]
            lilypond_multiplier = None
        elif len(args) == 2:
            written_duration, lilypond_multiplier = args
        else:
            raise ValueError('can not initialize rest from "%s".' % str(args))

        Leaf.__init__(self, written_duration, lilypond_multiplier)
        self._initialize_keyword_values(**kwargs)

    ### PRIVATE PROPERTIES ###

    @property
    def _body(self):
        '''Read-only body of rest.
        '''
        result = ''
        vertical_positioning_pitch = getattr(self, '_vertical_positioning_pitch', None)
        if vertical_positioning_pitch:
            result += str(vertical_positioning_pitch)
        else:
            result += 'r'
        result += str(self._formatted_duration)
        if vertical_positioning_pitch:
            result += r' \rest'
        return [result]

    @property
    def _compact_representation(self):
        return 'r%s' % self._formatted_duration



def get_tempo_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Get tempo marks attached to `component`::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)

    ::

        >>> contexttools.TempoMark(Duration(1, 8), 52)(staff)
        TempoMark(Duration(1, 8), 52)(Staff{4})

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                \tempo 8=52
                c'8
                d'8
                e'8
                f'8
            }
        >>

    ::

        >>> contexttools.get_tempo_marks_attached_to_component(staff)
        (TempoMark(Duration(1, 8), 52)(Staff{4}),)

    Return tuple of zero or more tempo marks.
    '''
    from abjad.tools import contexttools

    return contexttools.get_context_marks_attached_to_component(component, klasses=(contexttools.TempoMark,))



import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class ImmutableAbjadObject(AbjadObject):
    '''.. versionadded:: 2.8

    Abstract base class from which all custom classes which also subclass
    immutable builtin classes, such as tuple and frozenset, should inherit.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta
    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        pass

    @abc.abstractmethod
    def __new__(klass, *args, **kwargs):
        pass

    ### PRIVATE READ-ONLY PROPERTIES ###

    @property
    def _positional_argument_names(self):
        initializer = type(self).__new__
        if initializer.func_defaults:
            keyword_argument_count = len(initializer.func_defaults)
        else:
            keyword_argument_count = 0
        initializer_code = initializer.func_code
        positional_argument_count = (initializer_code.co_argcount - keyword_argument_count - 1)
        start_index, stop_index = 1, 1 + positional_argument_count
        return initializer_code.co_varnames[start_index:stop_index]

    ### PRIVATE METHODS ###

    @classmethod
    def _get_keyword_argument_names(cls):
        if hasattr(cls.__new__, 'func_defaults'):
            initializer = cls.__new__
            if initializer.func_defaults:
                keyword_argument_count = len(initializer.func_defaults)
                initializer_code = initializer.func_code
                positional_argument_count = (
                    initializer_code.co_argcount - keyword_argument_count - 1)
                start_index = 1 + positional_argument_count
                stop_index = start_index + keyword_argument_count
                return initializer_code.co_varnames[start_index:stop_index]
            else:
                return ()
        return ()



import re


space_delimited_lowercase_regex = re.compile('^(([a-z,0-9]+[ ]+)*[a-z,0-9]+)?$')

def is_space_delimited_lowercase_string(expr):
    r'''.. versionadded:: 2.5

    True when `expr` is a string and is space-delimited lowercase::

        >>> stringtools.is_space_delimited_lowercase_string('foo bar')
        True
    
    False otherwise::

        >>> stringtools.is_space_delimited_lowercase_string('foo_bar')
        False

    Return boolean.

    .. versionchanged:: 2.9
        renamed ``iotools.is_space_delimited_lowercase_string()`` to
        ``stringtools.is_space_delimited_lowercase_string()``.
    '''

    if not isinstance(expr, str):
        return False

    return bool(space_delimited_lowercase_regex.match(expr)) 



from abjad import *


def test_MultipartBeamSpanner___init___01():
    '''Init empty multipart beam spanner.
    '''

    beam = beamtools.MultipartBeamSpanner()
    assert isinstance(beam, beamtools.MultipartBeamSpanner)



from abjad.tools import markuptools
from abjad.tools import schemetools


def make_solid_text_spanner_below_with_nib_at_right(left_text, components=None):
    r'''.. versionadded:: 2.0

    Span `components` with text spanner.
    Position spanner below staff and configure with `left_text`,
    solid line and upward-pointing nib at right:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.make_solid_text_spanner_below_with_nib_at_right(
        ...     'foo', staff[:])
        TextSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup { foo }
            \override TextSpanner #'bound-details #'right #'text = \markup { \draw-line #'(0 . 1) }
            \override TextSpanner #'bound-details #'right-broken #'text = ##f
            \override TextSpanner #'dash-fraction = #1
            \override TextSpanner #'direction = #down
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details #'left #'text
            \revert TextSpanner #'bound-details #'right #'text
            \revert TextSpanner #'bound-details #'right-broken #'text
            \revert TextSpanner #'dash-fraction
            \revert TextSpanner #'direction
        }

    ::

        >>> show(staff) # doctest: +SKIP

    .. versionchanged:: 2.0
        renamed ``spanners.solid_text_spanner_below_with_nib_at_right()`` to
        ``spannertools.make_solid_text_spanner_below_with_nib_at_right()``.
    '''
    from abjad.tools import spannertools

    text_spanner = spannertools.TextSpanner(components)
    left_text = markuptools.Markup(left_text)
    text_spanner.override.text_spanner.bound_details__left__text = left_text
    right_text = markuptools.Markup(markuptools.MarkupCommand('draw-line', schemetools.SchemePair(0, 1)))
    text_spanner.override.text_spanner.bound_details__right__text = right_text
    text_spanner.override.text_spanner.bound_details__right_broken__text = False
    text_spanner.override.text_spanner.dash_fraction = 1
    text_spanner.override.text_spanner.direction = Down

    return text_spanner



def report_container_modifications(container):
    r'''Report `container` modifications as string:

    ::

        >>> container = Container("c'8 d'8 e'8 f'8")
        >>> container.override.note_head.color = 'red'
        >>> container.override.note_head.style = 'harmonic'

    ::

        >>> f(container)
        {
            \override NoteHead #'color = #red
            \override NoteHead #'style = #'harmonic
            c'8
            d'8
            e'8
            f'8
            \revert NoteHead #'color
            \revert NoteHead #'style
        }

    ::

        >>> string = containertools.report_container_modifications(container)

    ::

        >>> print string
        {
            \override NoteHead #'color = #red
            \override NoteHead #'style = #'harmonic
            %%% 4 components omitted %%%
            \revert NoteHead #'color
            \revert NoteHead #'style
        }

    Return string.
    '''
    from abjad.tools import containertools
    from abjad.tools import formattools

    assert isinstance(container, containertools.Container)

    fc = formattools.get_all_format_contributions(container)

    result = []

    result.extend(container._get_format_contributions_for_slot('before', fc))
    result.extend(container._get_format_contributions_for_slot('open brackets', fc))
    result.extend(container._get_format_contributions_for_slot('opening', fc))
    result.append('\t%%%%%% %s components omitted %%%%%%' % len(container))
    result.extend(container._get_format_contributions_for_slot('closing', fc))
    result.extend(container._get_format_contributions_for_slot('close brackets', fc))
    result.extend(container._get_format_contributions_for_slot('after', fc))

    result = '\n'.join(result)

    return result



def durations_to_integers(durations):
    '''Change `durations` to integers:

    ::

        >>> durations = [Duration(2, 4), 3, '8.', (5, 16)]
        >>> for integer in durationtools.durations_to_integers(durations):
        ...     integer
        ...
        8
        48
        3
        5

    Return new object of `durations` type.
    '''
    from abjad.tools import durationtools

    # change to nonreduced fracxtions
    nonreduced_fractions = durationtools.durations_to_nonreduced_fractions_with_common_denominator(
        durations)

    # find common denominator
    common_denominator = nonreduced_fractions[0].denominator

    # change to integers
    nonreduced_fractions = [
        common_denominator * nonreduced_fraction for nonreduced_fraction in nonreduced_fractions]
    fractions = [nonreduced_fraction.reduce() for nonreduced_fraction in nonreduced_fractions]
    assert all([fraction.denominator == 1 for fraction in fractions])
    integers = [fraction.numerator for fraction in fractions]
    
    # return integers
    return integers



from abjad.tools.abctools import AbjadObject
from abjad.tools import durationtools
from abjad.tools import tempotools


class SpacingIndication(AbjadObject):
    '''Spacing indication token.

    LilyPond ``Score.proportionalNotationDuration``
    will equal ``proportional_notation_duration`` when tempo
    equals ``tempo_indication``.

    Initialize from tempo mark and proportional notation duration::

        >>> tempo = contexttools.TempoMark(Duration(1, 8), 44)
        >>> indication = layouttools.SpacingIndication(tempo, Duration(1, 68))
        
    ::

        >>> indication
        SpacingIndication(TempoMark(Duration(1, 8), 44), Duration(1, 68))

    Initialize from constants::

        >>> layouttools.SpacingIndication(((1, 8), 44), (1, 68))
        SpacingIndication(TempoMark(Duration(1, 8), 44), Duration(1, 68))

    Initialize from other spacing indication::

        >>> layouttools.SpacingIndication(indication)
        SpacingIndication(TempoMark(Duration(1, 8), 44), Duration(1, 68))

    Spacing indications are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = (
        ((1, 8), 44), 
        (1, 68),
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import contexttools
        if len(args) == 1 and isinstance(args[0], type(self)):
            object.__setattr__(self, '_tempo_indication', args[0].tempo_indication)
            object.__setattr__(
                self, '_proportional_notation_duration', args[0].proportional_notation_duration)
        elif len(args) == 2:
            tempo_indication = contexttools.TempoMark(args[0])
            proportional_notation_duration = durationtools.Duration(args[1])
            #object.__setattr__(self, '_tempo_indication', args[0])
            #object.__setattr__(self, '_proportional_notation_duration', args[1])
            object.__setattr__(self, '_tempo_indication', tempo_indication)
            object.__setattr__(self, '_proportional_notation_duration', proportional_notation_duration)
        else:
            raise ValueError('can not initialize spacing indication from {!r}'.format(args))

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        '''Spacing indications compare equal when normalized spacing durations compare equal.
        '''
        if isinstance(expr, SpacingIndication):
            if self.normalized_spacing_duration == expr.normalized_spacing_duration:
                return True
        return False

    def __ne__(self, expr):
        '''Spacing indications compare unequal when normalized spacing durations compare unequal.
        '''
        return not self == expr

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _positional_argument_names(self):
        return ('_tempo_indication', '_proportional_notation_duration')

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def normalized_spacing_duration(self):
        '''Read-only proportional notation duration at 60 MM.
        '''
        indication = self.tempo_indication
        duration = self.proportional_notation_duration
        scalar = indication.duration / indication.units_per_minute * 60 / durationtools.Duration(1, 4)
        return scalar * self.proportional_notation_duration

    @property
    def proportional_notation_duration(self):
        '''LilyPond proportional notation duration context setting.
        '''
        return self._proportional_notation_duration

    @property
    def tempo_indication(self):
        '''Abjad tempo indication object.
        '''
        return self._tempo_indication



from abjad import *
import py


def test_TempoMark_attach_01():

    score = Score(r"\new Staff { c'' d'' e'' f'' } \new Staff { c' d' e' f' }")
    contexttools.TempoMark((1, 8), 52)(score[0][0])
    
    assert py.test.raises(ExtraMarkError, 'contexttools.TempoMark((1, 8), 52)(score[0][0])')
    assert py.test.raises(ExtraMarkError, 'contexttools.TempoMark((1, 8), 52)(score[1][0])')



from abjad.tools import pitchtools
from abjad.tools.quantizationtools.QEvent import QEvent


class PitchedQEvent(QEvent):
    '''A ``QEvent`` which indicates the onset of a period of pitched material
    in a ``QEventSequence``:

    ::

        >>> pitches = [0, 1, 4]
        >>> q_event = quantizationtools.PitchedQEvent(1000, pitches)
        >>> q_event
        quantizationtools.PitchedQEvent(
            durationtools.Offset(1000, 1),
            (NamedChromaticPitch("c'"), NamedChromaticPitch("cs'"), NamedChromaticPitch("e'")),
            attachments=()
            )

    Return ``PitchedQEvent`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_attachments', '_index', '_offset', '_pitches')

    ### INITIALIZER ###

    def __init__(self, offset, pitches, attachments=None, index=None):
        QEvent.__init__(self, offset, index=index)
        pitches = tuple([pitchtools.NamedChromaticPitch(x) for x in pitches])
        if attachments is None:
            attachments = ()
        else:
            attachments = tuple(attachments)
        self._pitches = pitches
        self._attachments = attachments

    ### SPECIAL METHODS ###

    def __eq__(self, other):
        if type(self) == type(other) and \
            self.offset == other.offset and \
            self.pitches == other.pitches and \
            self.attachments == other.attachments and \
            self.index == other.index:
            return True
        return False

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def attachments(self):
        return self._attachments

    @property
    def pitches(self):
        return self._pitches



from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__containers__parallel_01():
    target = Container()
    target.is_parallel = True
    parser = LilyPondParser()
    result = parser(target.lilypond_format)
    assert target.lilypond_format == result.lilypond_format and target is not result



def space_delimited_lowercase_to_uppercamelcase(string):
    '''.. versionadded:: 2.6

    Change space-delimited lowercase `string` to uppercamelcase::

        >>> string = 'bass figure alignment positioning'
        >>> stringtools.space_delimited_lowercase_to_uppercamelcase(string)
        'BassFigureAlignmentPositioning'

    Return string.

    .. versionchanged:: 2.9
        renamed ``iotools.space_delimited_lowercase_to_uppercamelcase()`` to
        ``stringtools.space_delimited_lowercase_to_uppercamelcase()``.
    '''

    parts = string.split(' ')
    parts = [part.title() for part in parts]
    return ''.join(parts)



def calculate_melodic_counterpoint_interval(pitch_carrier_1, pitch_carrier_2):
    '''.. versionadded:: 2.0

    Calculate melodic counterpoint interval `pitch_carrier_1` to
    `pitch_carrier_2`::

        >>> pitchtools.calculate_melodic_counterpoint_interval(
        ... pitchtools.NamedChromaticPitch(-2), pitchtools.NamedChromaticPitch(12))
        MelodicCounterpointInterval(+9)

    Return melodic counterpoint interval.
    '''
    from abjad.tools import pitchtools

    # get melodic diatonic interval
    mdi = pitchtools.calculate_melodic_diatonic_interval(pitch_carrier_1, pitch_carrier_2)

    # return melodic counterpoint interval
    return mdi.melodic_counterpoint_interval



from abjad import *
import py


def test_OrdinalConstant___builtins___01():

    assert Left < Right
    assert Down < Center < Up


def test_OrdinalConstant___builtins___02():

    assert py.test.raises(Exception, 'Down < Right')



import fractions
import math
from abjad.tools import durationtools
from abjad.tools import mathtools


def rewrite_integer_tempo(integer_tempo, maximum_numerator=None, maximum_denominator=None):
    r'''.. versionadded:: 2.0

    Rewrite `integer_tempo`.

    Allow no tempo less than half `integer_tempo` or greater than double `integer_tempo`::

        >>> pairs = tempotools.rewrite_integer_tempo(
        ...     58, maximum_numerator=8, maximum_denominator=8)

    ::

        >>> for pair in pairs:
        ...     pair
        ...
        (Multiplier(1, 2), 29)
        (Multiplier(1, 1), 58)
        (Multiplier(3, 2), 87)
        (Multiplier(2, 1), 116)

    Return list.
    '''

    # find divisors
    divisors = mathtools.divisors(integer_tempo)
    if maximum_denominator is not None:
      divisors = [x for x in divisors if x <= maximum_denominator]

    # make pairs
    pairs = []
    for divisor in divisors:
      start = int(math.ceil(divisor / 2.0))
      stop = 2 * divisor
      numerators = range(start, stop + 1)
      if maximum_numerator is not None:
            numerators = [x for x in numerators if x <= maximum_numerator]
      for numerator in numerators:
            multiplier = durationtools.Multiplier(numerator, divisor)
            new_tempo = fractions.Fraction(multiplier * integer_tempo)
            assert mathtools.is_integer_equivalent_number(new_tempo)
            new_tempo = int(new_tempo)
            pair = (multiplier, new_tempo)
            if pair not in pairs:
                pairs.append(pair)

    # sort pairs
    pairs.sort()

    # return pairs
    return pairs



from abjad.tools import durationtools
from abjad.tools import notetools
from abjad.tools.rhythmtreetools import RhythmTreeNode


class QGridLeaf(RhythmTreeNode):
    '''A leaf in a ``QGrid`` structure:

    ::

        >>> leaf = quantizationtools.QGridLeaf()

    ::

        >>> leaf
        QGridLeaf(
            duration=Duration(1, 1),
            is_divisible=True
            )

    Used internally by ``QGrid``.

    Return ``QGridLeaf`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    #__slots__ = ('_duration', '_is_divisible', '_offset', '_offsets_are_current',
    #    '_parent', '_q_event_proxies')

    ### INITIALIZER ###

    def __init__(self, duration=1, q_event_proxies=None, is_divisible=True):
        from abjad.tools import quantizationtools
        RhythmTreeNode.__init__(self, duration)
        if q_event_proxies is None:
            self._q_event_proxies = []
        else:
            assert all([isinstance(x, quantizationtools.QEventProxy) for x in q_event_proxies])
            self._q_event_proxies = list(q_event_proxies)            
        self._is_divisible = bool(is_divisible)

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.duration
        return notetools.make_notes(0, total_duration)

    def __deepcopy__(self, memo):
        return type(self)(*self.__getnewargs__())

    def __eq__(self, other):
        if type(self) == type(other):
            if self.duration == other.duration:
                if self.q_event_proxies == other.q_event_proxies:
                    if self._is_divisible == other.is_divisible:
                        return True
        return False

    def __getnewargs__(self):
       return (self.duration, tuple(self.q_event_proxies), self.is_divisible)

    ### READ-ONLY PRIVATE PROPERTIES ###
        
    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.duration)]

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def graphviz_graph(self):
        graph = documentationtools.GraphvizGraph(name='G')
        node = documentationtools.GraphvizNode(
            attributes={
                label: str(self.duration),
                shape: 'box'
            }
            )
        graph.append(node)
        return graph

    @property
    def preceding_q_event_proxies(self):
        return [x for x in self._q_event_proxies if x.offset < self.start_offset]

    @property
    def q_event_proxies(self):
        return self._q_event_proxies

    @property
    def rtm_format(self):
        return str(self.duration)

    @property
    def succeeding_q_event_proxies(self):
        return [x for x in self._q_event_proxies if self.start_offset <= x.offset]

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def is_divisible():
        def fget(self):
            '''Flag for whether the node may be further divided under some search tree.'''
            return self._is_divisible
        def fset(self, arg):
            self._is_divisible = bool(arg)
        return property(**locals())



from abjad import *


def test_pitchtools_split_chromatic_pitch_class_name_01():

    assert pitchtools.split_chromatic_pitch_class_name('c') == ('c', '')
    assert pitchtools.split_chromatic_pitch_class_name('cs') == ('c', 's')
    assert pitchtools.split_chromatic_pitch_class_name('d') == ('d', '')
    assert pitchtools.split_chromatic_pitch_class_name('ds') == ('d', 's')



from abjad.tools import componenttools


def iterate_thread_in_expr(expr, klass, containment_signature, reverse=False):
    r'''.. versionadded:: 2.10

    Yield left-to-right instances of `klass` in `expr` with `containment_signature`::

        >>> container = Container(Voice(notetools.make_repeated_notes(2)) * 2)
        >>> container.is_parallel = True
        >>> container[0].name = 'voice 1'
        >>> container[1].name = 'voice 2'
        >>> staff = Staff(container * 2)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(staff)

    ::

        >>> f(staff)
        \new Staff {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
            <<
                \context Voice = "voice 1" {
                    g'8
                    a'8
                }
                \context Voice = "voice 2" {
                    b'8
                    c''8
                }
            >>
        }

    ::

        >>> signature = staff.leaves[0].parentage.containment_signature
        >>> for x in iterationtools.iterate_thread_in_expr(staff, Note, signature):
        ...     x
        ...
        Note("c'8")
        Note("d'8")
        Note("g'8")
        Note("a'8")

    Return generator.
    '''
    from abjad.tools import iterationtools

    if isinstance(expr, klass) and expr.parentage.containment_signature == containment_signature:
        yield expr

    if not reverse:
        if isinstance(expr, (list, tuple)):
            for m in expr:
                for x in iterationtools.iterate_thread_in_expr(
                    m, klass, containment_signature):
                    yield x
        if hasattr(expr, '_music'):
            for m in expr._music:
                for x in iterationtools.iterate_thread_in_expr(
                    m, klass, containment_signature):
                    yield x
    else:
        if isinstance(expr, (list, tuple)):
            for m in reversed(expr):
                for x in iterationtools.iterate_thread_in_expr(
                    m, klass, containment_signature, reverse=True):
                    yield x
        if hasattr(expr, '_music'):
            for m in reversed(expr._music):
                for x in iterationtools.iterate_thread_in_expr(
                    m, klass, containment_signature, reverse=True):
                    yield x



from abjad import *


def test_markuptools_make_big_centered_page_number_markup_01():

    markup = markuptools.make_big_centered_page_number_markup()

    r'''
    \markup {
        \fill-line
            {
                \bold
                    \fontsize
                        #3
                        \concat
                            {
                                \on-the-fly
                                    #print-page-number-check-first
                                    \fromproperty
                                        #'page:page-number-string
                            }
            }
        }
    '''

    assert markup.indented_lilypond_format == "\\markup {\n\t\\fill-line\n\t\t{\n\t\t\t\\bold\n\t\t\t\t\\fontsize\n\t\t\t\t\t#3\n\t\t\t\t\t\\concat\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\\on-the-fly\n\t\t\t\t\t\t\t\t#print-page-number-check-first\n\t\t\t\t\t\t\t\t\\fromproperty\n\t\t\t\t\t\t\t\t\t#'page:page-number-string\n\t\t\t\t\t\t}\n\t\t}\n\t}"



def report_spanner_format_contributions(spanner, screen=True):
    r'''.. versionadded:: 2.9

    Report spanner format contributions for every leaf to which spanner attaches.

        >>> staff = Staff("c8 d e f")
        >>> spanner = beamtools.BeamSpanner(staff[:])

    ::

        >>> formattools.report_spanner_format_contributions(spanner)
        c8  before: []
             after: []
             right: ['[']
        <BLANKLINE>
        d8  before: []
             after: []
             right: []
        <BLANKLINE>
        e8  before: []
             after: []
             right: []
        <BLANKLINE>
        f8  before: []
             after: []
             right: [']']

    Return none or return string.
    '''
    result = ''
    for leaf in spanner.leaves:
        result += str(leaf)
        result += '\tbefore: %s\n' % spanner._format_before_leaf(leaf)
        result += '\t after: %s\n' % spanner._format_after_leaf(leaf)
        result += '\t right: %s\n' % spanner._format_right_of_leaf(leaf)
        result += '\n'
    if result[-1] == '\n':
        result = result[:-1]
    if screen:
        print result
    else:
        return result



from abjad import *


def test_tietools_iterate_nontrivial_tie_chains_in_expr_01():

    staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

    tie_chains = list(tietools.iterate_nontrivial_tie_chains_in_expr(staff, reverse=True))

    assert tie_chains[0].leaves == staff.leaves[-2:]
    assert tie_chains[1].leaves == staff.leaves[:2]


def test_tietools_iterate_nontrivial_tie_chains_in_expr_02():

    staff = Staff(r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16")

    tie_chains = list(tietools.iterate_nontrivial_tie_chains_in_expr(staff))

    assert tie_chains[0].leaves == staff.leaves[:2]
    assert tie_chains[1].leaves == staff.leaves[-2:]



def get_down_markup_attached_to_component(component):
    '''.. versionadded:: 2.0

    Get down-markup attached to component::

        >>> chord = Chord([-11, 2, 5], (1, 4))

    ::

        >>> markuptools.Markup('UP', Up)(chord)
        Markup(('UP',), direction=Up)(<cs d' f'>4)

    ::

        >>> markuptools.Markup('DOWN', Down)(chord)
        Markup(('DOWN',), direction=Down)(<cs d' f'>4)

    ::

        >>> markuptools.get_down_markup_attached_to_component(chord)
        (Markup(('DOWN',), direction=Down)(<cs d' f'>4),)

    Return tuple of zero or more markup objects.
    '''
    from abjad.tools import markuptools

    result = []

    markups = markuptools.get_markup_attached_to_component(component)
    for markup in markups:
        if markup.direction is Down:
            result.append(markup)

    return tuple(result)



def group_overlapping_intervals_and_yield_groups(intervals):
    '''Group overlapping intervals in `intervals`.

    Yield TimeIntervalTrees.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        yield timeintervaltools.TimeIntervalTree([])
        return

    groups = []
    group = [tree[0]]

    # start = group[0].start
    stop = group[0].stop

    for i in range(1, len(tree)):
        if tree[i].start < stop:
            group.append(tree[i])
            if stop < tree[i].stop:
                stop = tree[i].stop
        else:
            groups.append(group)
            group = [tree[i]]
            # start = group[0].start
            stop = group[0].stop

    if group not in groups:
        groups.append(group)

    for group in groups:
        yield timeintervaltools.TimeIntervalTree(group)



from abjad import *
import py


def test_FixedDurationContainer_format_01():

    container = containertools.FixedDurationContainer((3, 8), "c'8 d'8 e'8")
    assert container.lilypond_format == "{\n\tc'8\n\td'8\n\te'8\n}"


def test_FixedDurationContainer_format_02():

    container = containertools.FixedDurationContainer((3, 8), "c'8 d'8")
    assert py.test.raises(Exception, 'container.format')


def test_FixedDurationContainer_format_03():

    container = containertools.FixedDurationContainer((3, 8), "c'8 d'8 e'8 f'8")
    assert py.test.raises(Exception, 'container.format')



from abjad import *


def test_Chord___repr___01():
    '''Chord repr is evaluable.
    '''

    chord_1 = Chord([3, 13, 17], (1, 4))
    chord_2 = eval(repr(chord_1))

    assert isinstance(chord_1, Chord)
    assert isinstance(chord_2, Chord)
    assert chord_1.lilypond_format == chord_2.lilypond_format
    assert chord_1 is not chord_2


def test_Chord___repr___02():
    chord = Chord('<c! e? g!? b>4')
    assert "Chord('{}')".format(chord.lilypond_format) == repr(chord)



import abc
import fractions
import inspect
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.datastructuretools.TreeNode import TreeNode


class RhythmTreeNode(TreeNode):
    '''Abstract base class of nodes in a rhythm tree structure.'''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, duration=1, name=None):
        TreeNode.__init__(self, name=name)
        self._duration = 0
        self._offset = durationtools.Offset(0)
        self._offsets_are_current = False
        self.duration = duration

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, pulse_duration):
        raise NotImplemented

    ### PRIVATE METHODS ###

    def _update_offsets_of_entire_tree(self):
        def recurse(container, current_offset):
            container._offset = current_offset
            container._offsets_are_current = True
            for child in container:
                if hasattr(child, 'children'):
                    current_offset = recurse(child, current_offset)
                else:
                    child._offset = current_offset
                    child._offsets_are_current = True
                    current_offset += child.prolated_duration
            return current_offset
        offset = durationtools.Offset(0)
        root = self.root
        if root is self and not hasattr(self, 'children'):
            self._offset = offset
            self._offsets_are_current = True
        else:
            recurse(root, offset)

    def _update_offsets_of_entire_tree_if_necessary(self):
        if not self._get_node_state_flags()['_offsets_are_current']:
            self._update_offsets_of_entire_tree()

    ### READ-ONLY PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _pretty_rtm_format_pieces(self):
        raise NotImplemented

    @property
    def _state_flag_names(self):
        return ('_offsets_are_current',)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def graphviz_format(self):
        return self.graphviz_graph.graphviz_format

    @abc.abstractproperty
    def graphviz_graph(self):
        raise NotImplemented

    @property
    def parentage_ratios(self):
        '''A sequence describing the relative durations of the nodes in a
        node's improper parentage.

        The first item in the sequence is the duration of the root node, and
        subsequent items are pairs of the duration of the next node in the
        parentage chain and the total duration of that node and its siblings:


        ::

            >>> a = rhythmtreetools.RhythmTreeContainer(duration=1)
            >>> b = rhythmtreetools.RhythmTreeContainer(duration=2)
            >>> c = rhythmtreetools.RhythmTreeLeaf(duration=3)
            >>> d = rhythmtreetools.RhythmTreeLeaf(duration=4)
            >>> e = rhythmtreetools.RhythmTreeLeaf(duration=5)

        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])

        ::

            >>> a.parentage_ratios
            (Duration(1, 1),)

        ::

            >>> b.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)))

        ::

            >>> c.parentage_ratios
            (Duration(1, 1), (Duration(3, 1), Duration(5, 1)))
        
        ::

            >>> d.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)), (Duration(4, 1), Duration(9, 1)))

        ::

            >>> e.parentage_ratios
            (Duration(1, 1), (Duration(2, 1), Duration(5, 1)), (Duration(5, 1), Duration(9, 1)))

        Return tuple.
        '''
        result = []
        node = self
        while node.parent is not None:
            result.append((node.duration, node.parent.contents_duration))
            node = node.parent
        result.append(node.duration)
        return tuple(reversed(result))

    @property
    def pretty_rtm_format(self):
        '''The node's pretty-printed RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> print tree.pretty_rtm_format
            (1 (
                (1 (
                    1
                    1))
                (1 (
                    1
                    1))))

        Return string.
        '''
        return '\n'.join(self._pretty_rtm_format_pieces)

    @property
    def prolated_duration(self):
        '''The prolated duration of the node:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree.prolated_duration
            Duration(1, 1)

        ::

            >>> tree[1].prolated_duration
            Duration(1, 2)

        ::

            >>> tree[1][1].prolated_duration
            Duration(1, 4)

        Return `Duration` instance.
        '''
        return self.prolation * self.duration

    @property
    def prolation(self):
        return mathtools.cumulative_products(self.prolations)[-1]

    @property
    def prolations(self):
        prolations = [durationtools.Multiplier(1)]
        improper_parentage = self.improper_parentage
        for child, parent in sequencetools.iterate_sequence_pairwise_strict(improper_parentage):
            prolations.append(durationtools.Multiplier(parent.duration, parent.contents_duration))
        return tuple(prolations)

    @abc.abstractproperty
    def rtm_format(self):
        '''The node's RTM format:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
            >>> tree.rtm_format
            '(1 ((1 (1 1)) (1 (1 1))))'

        Return string.
        '''
        raise NotImplemented

    @property
    def start_offset(self):
        '''The starting offset of a node in a rhythm-tree relative the root:

        ::

            >>> rtm = '(1 ((1 (1 1)) (1 (1 1))))'
            >>> tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]

        ::

            >>> tree.start_offset
            Offset(0, 1)

        ::

            >>> tree[1].start_offset
            Offset(1, 2)

        ::

            >>> tree[0][1].start_offset
            Offset(1, 4)

        Return Offset instance.
        '''
        self._update_offsets_of_entire_tree_if_necessary()
        return self._offset

    @property
    def stop_offset(self):
        '''The stopping offset of a node in a rhythm-tree relative the root.
        '''
        return self.start_offset + self.prolated_duration

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def duration():
        def fget(self):
            '''The node's duration in pulses:

            ::

                >>> node = rhythmtreetools.RhythmTreeLeaf(duration=1)
                >>> node.duration
                Duration(1, 1)

            ::

                >>> node.duration = 2
                >>> node.duration
                Duration(2, 1)

            Return int.
            '''
            return self._duration
        def fset(self, arg):
            if not isinstance(arg, fractions.Fraction):
                arg = durationtools.Duration(arg)
            assert 0 < arg
            self._duration = arg
            self._mark_entire_tree_for_later_update()
        return property(**locals())




import abc

from abjad.tools.pitchtools.ObjectSet import ObjectSet


class PitchClassObjectSet(ObjectSet):
    '''.. versionadded:: 2.0

    Pitch-class set base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INITIALIZER ###
    
    @abc.abstractmethod
    def __new__(self):
        pass



import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class TimeRelation(AbjadObject):
    r'''.. versionadded:: 2.11

    Time relation:

    ::

        >>> timespan_1 = timespantools.Timespan(0, 10)
        >>> timespan_2 = timespantools.Timespan(5, 15)
        >>> time_relation = timerelationtools.timespan_2_starts_during_timespan_1(
        ...     timespan_1=timespan_1, timespan_2=timespan_2, hold=True)


    ::

        >>> z(time_relation)
        timerelationtools.TimespanTimespanTimeRelation(
            'timespan_1.start <= timespan_2.start < timespan_1.stop',
            timespan_1=timespantools.Timespan(
                start_offset=durationtools.Offset(0, 1),
                stop_offset=durationtools.Offset(10, 1)
                ),
            timespan_2=timespantools.Timespan(
                start_offset=durationtools.Offset(5, 1),
                stop_offset=durationtools.Offset(15, 1)
                )
            )

    Time relations are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, template):
        assert isinstance(template, str), repr(template)
        self._template = template

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self):
        '''Evaluate time relation:

        ::

            >>> time_relation()
            True
    
        Return boolean.
        '''
        pass

    @abc.abstractmethod
    def __eq__(self, expr):
        '''True when `expr` is a time relation with
        template and both terms equal to time relation.
        Otherwise false.
    
        Return boolean.
        '''
        pass

    ### PRIVATE METHODS ###

    def _get_expr_offsets(self, expr, score_specification=None, context_name=None):
        if hasattr(expr, 'offsets'):
            return expr.offsets
        elif hasattr(expr, 'get_offsets'):
            return expr.get_offsets(score_specification, context_name)
        else:
            raise ValueError('{!r} has neither offsets property nor get_offsets() method.'.format(expr))

    ### READ-ONLY PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def is_fully_loaded(self):
        '''True when both time relation terms are not none.
        Otherwise false:

        ::

            >>> time_relation.is_fully_loaded
            True
    
        Return boolean.
        '''
        pass

    @abc.abstractproperty
    def is_fully_unloaded(self):
        '''True when both time relation terms are none.
        Otherwise false:

        ::

            >>> time_relation.is_fully_unloaded
            False
    
        Return boolean.
        '''
        pass

    @property
    def storage_format(self):
        '''Time relation storage format:

        ::

            >>> z(time_relation)
            timerelationtools.TimespanTimespanTimeRelation(
                'timespan_1.start <= timespan_2.start < timespan_1.stop',
                timespan_1=timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(10, 1)
                    ),
                timespan_2=timespantools.Timespan(
                    start_offset=durationtools.Offset(5, 1),
                    stop_offset=durationtools.Offset(15, 1)
                    )
                )

        Return string.
        '''
        return AbjadObject.storage_format.fget(self)

    @property
    def template(self):
        '''Time relation template:
    
        ::

            >>> time_relation.template
            'timespan_1.start <= timespan_2.start < timespan_1.stop'

        Return string.
        '''
        return self._template

    ### PUBLIC METHODS ###

    def new(self, **kwargs):
        '''Inialize new time relation with keyword arguments optionally changed:

        ::

            >>> time_relation = timerelationtools.timespan_2_stops_when_timespan_1_starts()
            >>> new_time_relation = time_relation.new(timespan_1=timespantools.Timespan(0, 5))

        ::

            >>> z(time_relation)
            timerelationtools.TimespanTimespanTimeRelation(
                'timespan_2.stop == timespan_1.start'
                )

        ::

            >>> z(new_time_relation)
            timerelationtools.TimespanTimespanTimeRelation(
                'timespan_2.stop == timespan_1.start',
                timespan_1=timespantools.Timespan(
                    start_offset=durationtools.Offset(0, 1),
                    stop_offset=durationtools.Offset(5, 1)
                    )
                )

        Return newly constructed time relation.
        '''
        keyword_argument_dictionary = self._keyword_argument_dictionary
        for key, value in kwargs.iteritems():
            keyword_argument_dictionary[key] = value
        result = type(self)(*self._positional_argument_values, **keyword_argument_dictionary)
        return result



def timespan_2_happens_during_timespan_1(timespan_1=None, timespan_2=None, hold=False):
    r'''.. versionadded:: 2.11

    Make time relation indicating that expression 2 happens during expression 1::

        >>> timerelationtools.timespan_2_happens_during_timespan_1()
        TimespanTimespanTimeRelation('timespan_1.start <= timespan_2.start <= timespan_2.stop <= timespan_1.stop')

    Evaluate whether timespan ``[7/8, 8/8)`` happens during timespan ``[1/2, 3/2)``::

        >>> timespan_1 = timespantools.Timespan(Offset(1, 2), Offset(3, 2))
        >>> timespan_2 = timespantools.Timespan(Offset(7, 8), Offset(8, 8))
        >>> timerelationtools.timespan_2_happens_during_timespan_1(
        ...     timespan_1=timespan_1, timespan_2=timespan_2)
        True

    Return time relation or boolean.
    '''
    from abjad.tools import timerelationtools

    time_relation = timerelationtools.TimespanTimespanTimeRelation(
        'timespan_1.start <= timespan_2.start <= timespan_2.stop <= timespan_1.stop',
        timespan_1=timespan_1, 
        timespan_2=timespan_2)
    
    if time_relation.is_fully_loaded and not hold: 
        return time_relation()
    else:
        return time_relation



from abjad import *


def test_pitchtools_set_written_pitch_of_pitched_components_in_expr_01():

    staff = Staff("c' d' e' f'")

    r'''
    \new Staff {
        c'4
        d'4
        e'4
        f'4
    }
    '''

    pitchtools.set_written_pitch_of_pitched_components_in_expr(staff)

    r'''
    \new Staff {
        c'4
        c'4
        c'4
        c'4
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\tc'4\n\tc'4\n\tc'4\n\tc'4\n}"



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools.quantizationtools.QSchema import QSchema


class BeatwiseQSchema(QSchema):
    '''Concrete ``QSchema`` subclass which treats "beats" as its time-step unit:

    ::

        >>> q_schema = quantizationtools.BeatwiseQSchema()

    Without arguments, it uses smart defaults:

    ::

        >>> q_schema
        quantizationtools.BeatwiseQSchema(
            beatspan=durationtools.Duration(1, 4),
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={   2: {   2: {   2: {   2: None}, 3: None}, 3: None, 5: None, 7: None},
                    3: {   2: {   2: None}, 3: None, 5: None},
                    5: {   2: None, 3: None},
                    7: {   2: None},
                    11: None,
                    13: None}
                ),
            tempo=contexttools.TempoMark(
                durationtools.Duration(1, 4),
                60
                ),
            )

    Each time-step in a ``BeatwiseQSchema`` is composed of three settings:

        * ``beatspan``
        * ``search_tree``
        * ``tempo``

    These settings can be applied as global defaults for the schema via keyword arguments,
    which persist until overridden:

    ::

        >>> beatspan = Duration(5, 16)
        >>> search_tree = quantizationtools.UnweightedSearchTree({7: None})
        >>> tempo = contexttools.TempoMark((1, 4), 54)
        >>> q_schema = quantizationtools.BeatwiseQSchema(
        ...     beatspan=beatspan,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     )

    The computed value at any non-negative time-step can be found by subscripting:

    ::

        >>> index = 0
        >>> for key, value in sorted(q_schema[index].items()): print '{}:'.format(key), value
        ...
        beatspan: 5/16
        search_tree: UnweightedSearchTree(
            definition={   7: None}
            )
        tempo: TempoMark(Duration(1, 4), 54)

    ::

        >>> index = 1000
        >>> for key, value in sorted(q_schema[index].items()): print '{}:'.format(key), value
        ...
        beatspan: 5/16
        search_tree: UnweightedSearchTree(
            definition={   7: None}
            )
        tempo: TempoMark(Duration(1, 4), 54)

    Per-time-step settings can be applied in a variety of ways.

    Instantiating the schema via ``*args`` with a series of either
    ``BeatwiseQSchemaItem`` instances, or dictionaries which could be used to
    instantiate ``BeatwiseQSchemaItem`` instances, will apply those settings 
    sequentially, starting from time-step ``0``:

    ::

        >>> a = {'beatspan': Duration(5, 32)}
        >>> b = {'beatspan': Duration(3, 16)}
        >>> c = {'beatspan': Duration(1, 8)}

    ::

        >>> q_schema = quantizationtools.BeatwiseQSchema(a, b, c)

    ::

        >>> q_schema[0]['beatspan']
        Duration(5, 32)

    ::

        >>> q_schema[1]['beatspan']
        Duration(3, 16)

    ::

        >>> q_schema[2]['beatspan']
        Duration(1, 8)

    ::

        >>> q_schema[3]['beatspan']
        Duration(1, 8)

    Similarly, instantiating the schema from a single dictionary, consisting of
    integer:specification pairs, or a sequence via ``*args`` of (integer,
    specification) pairs, allows for applying settings to  non-sequential
    time-steps:

    ::

        >>> a = {'search_tree': quantizationtools.UnweightedSearchTree({2: None})}
        >>> b = {'search_tree': quantizationtools.UnweightedSearchTree({3: None})}

    ::

        >>> settings = {
        ...     2: a,
        ...     4: b,
        ... }

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(settings)

    ::

        >>> q_schema[0]['search_tree']
        UnweightedSearchTree(
            definition={   2: {   2: {   2: {   2: None}, 3: None}, 3: None, 5: None, 7: None},
                3: {   2: {   2: None}, 3: None, 5: None},
                5: {   2: None, 3: None},
                7: {   2: None},
                11: None,
                13: None}
            )

    ::

        >>> q_schema[1]['search_tree']
        UnweightedSearchTree(
            definition={   2: {   2: {   2: {   2: None}, 3: None}, 3: None, 5: None, 7: None},
                3: {   2: {   2: None}, 3: None, 5: None},
                5: {   2: None, 3: None},
                7: {   2: None},
                11: None,
                13: None}
            )

    ::

        >>> q_schema[2]['search_tree']
        UnweightedSearchTree(
            definition={   2: None}
            )

    ::

        >>> q_schema[3]['search_tree']
        UnweightedSearchTree(
            definition={   2: None}
            )

    ::

        >>> q_schema[4]['search_tree']
        UnweightedSearchTree(
            definition={   3: None}
            )

    ::

        >>> q_schema[1000]['search_tree'] 
        UnweightedSearchTree(
            definition={   3: None}
            )

    The following is equivalent to the above schema definition:

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(
        ...     (2, {'search_tree': quantizationtools.UnweightedSearchTree({2: None})}),
        ...     (4, {'search_tree': quantizationtools.UnweightedSearchTree({3: None})}),
        ...     )

    Return ``BeatwiseQSchema`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_beatspan', '_items', '_lookups', '_search_tree', '_tempo',)

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools import quantizationtools

        self._beatspan = durationtools.Duration(
            kwargs.get('beatspan',
                (1, 4)))

        search_tree = kwargs.get('search_tree', quantizationtools.UnweightedSearchTree()) 
        assert isinstance(search_tree, quantizationtools.SearchTree)
        self._search_tree = search_tree

        self._tempo = contexttools.TempoMark(
            kwargs.get('tempo',
                ((1, 4), 60)))

        QSchema.__init__(self, *args, **kwargs)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _keyword_argument_names(self):
        return ('beatspan', 'search_tree', 'tempo')

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        '''The default beatspan.'''
        return self._beatspan

    @property
    def item_klass(self):
        '''The schema's item class.'''
        from abjad.tools import quantizationtools
        return quantizationtools.BeatwiseQSchemaItem

    @property
    def target_item_klass(self):
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetBeat

    @property
    def target_klass(self):
        from abjad.tools import quantizationtools
        return quantizationtools.BeatwiseQTarget



def yield_topmost_components_of_klass_grouped_by_type(expr, klass):
    r'''.. versionadded:: 2.0

    Example 1. Yield runs of topmost notes in `expr`::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 r8 } \times 2/3 { r8 <e' g'>8 <f' a'>8 }")
        >>> staff.extend("g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8")

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                r8
            }
            \times 2/3 {
                r8
                <e' g'>8
                <f' a'>8
            }
            g'8
            a'8
            r8
            r8
            <b' d''>8
            <c'' e''>8
        }

    ::
    
        >>> for group in componenttools.yield_topmost_components_of_klass_grouped_by_type(
        ...     staff, Note):
        ...     group
        (Note("g'8"), Note("a'8"))

    Example 2. Yield runs of notes at all levels in `expr`::

        >>> leaves = iterationtools.iterate_leaves_in_expr(staff)

    ::

        >>> for group in componenttools.yield_topmost_components_of_klass_grouped_by_type(
        ...     leaves, Note):
        ...     group
        (Note("c'8"), Note("d'8"))
        (Note("g'8"), Note("a'8"))

    Return generator.
    '''
    from abjad.tools import componenttools

    for group in componenttools.yield_topmost_components_grouped_by_type(expr):
        if isinstance(group[0], klass):
            yield group



from abjad import *
from abjad.tools import iotools


def test_iotools_get_last_output_file_name_01():

    last_output_file_name = iotools.get_last_output_file_name()

    assert isinstance(last_output_file_name, (str, type(None)))
    if isinstance(last_output_file_name, str):
        if last_output_file_name.endswith('.ly'):
            assert len(last_output_file_name) == 7
        elif last_output_file_name.endswith('.pdf'):
            assert len(last_output_file_name) == 8
        elif last_output_file_name.endswith('.mid'):
            assert len(last_output_file_name) == 8
        elif last_output_file_name.endswith('.midi'):
            assert len(last_output_file_name) == 9
        else:
            raise Exception



from abjad.tools import durationtools


def round_interval_bounds_to_nearest_multiple_of_rational(intervals, duration):
    '''Round all start and stop offsets of `intervals` to the nearest multiple of `duration`.

    If both start and stop of an interval collapse on the same offset, that interval's stop
    will be adjusted to the next larger multiple of `duration`.

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    duration = durationtools.Duration(duration)
    assert 0 < duration

    if not tree:
        return tree

    intervals = []
    for interval in tree:
        start = durationtools.Offset(int(round(interval.start / duration))) * duration
        stop = durationtools.Offset(int(round(interval.stop / duration))) * duration
        if start == stop:
            stop = start + duration
        intervals.append(interval.shift_to_rational(start).scale_to_rational(stop - start))

    return timeintervaltools.TimeIntervalTree(intervals)



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools._KeyboardInstrument import _KeyboardInstrument


class Harpsichord(_KeyboardInstrument):
    r'''.. versionadded:: 2.5

    Abjad model of the harpsichord::

        >>> piano_staff = scoretools.PianoStaff([Staff("c'8 d'8 e'8 f'8"), Staff("c'4 b4")])

    ::

        >>> instrumenttools.Harpsichord()(piano_staff)
        Harpsichord()(PianoStaff<<2>>)

    ::

        >>> f(piano_staff)
        \new PianoStaff <<
            \set PianoStaff.instrumentName = \markup { Harpsichord }
            \set PianoStaff.shortInstrumentName = \markup { Hpschd. }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                c'4
                b4
            }
        >>

    ::

        >>> show(piano_staff) # doctest: +SKIP

    The harpsichord targets piano staff context by default.

    Return instrument.
    '''

    def __init__(self, target_context=None, **kwargs):
        if target_context is None:
            target_context = scoretools.PianoStaff
        _KeyboardInstrument.__init__(self, target_context=target_context, **kwargs)    
        self._default_instrument_name = 'harpsichord'
        self._default_performer_names.append('harpsichordist')
        self._default_short_instrument_name = 'hpschd.'
        self._is_primary_instrument = True
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange(-24, 36)



from abjad.tools.abctools import AbjadObject


class Infinity(AbjadObject):
    '''Object-oriented infinity.

    All numbers compare less than infinity:

    ::

        >>> 9999999 < Infinity
        True

    ::

        >>> 2**38 < Infinity
        True

    Infinity compares equal to itself:

    ::

        >>> Infinity == Infinity
        True

    Negative infinity compares less than infinity:

    ::

        >>> NegativeInfinity < Infinity
        True

    Infinity is initialized at start-up and is available in the global Abjad namespace.
    '''

    ### INTIALIZER ###

    def __init__(self):
        self._value = float('infinity')

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        if isinstance(expr, type(self)):
            return self._value == expr._value
        return False

    def __ge__(self, expr):
        return self._value >= expr

    def __gt__(self, expr):
        return self._value > expr

    def __le__(self, expr):
        return self._value <= expr

    def __lt__(self, expr):
        return self._value < expr

    def __repr__(self):
        return self._class_name



import fractions
from abjad.tools import durationtools
from abjad.tools import mathtools


def make_accelerating_notes_with_lilypond_multipliers(pitches, total, start, stop, exp='cosine', written='8'):
    '''Make accelerating notes with LilyPond multipliers::

        >>> pitches = [1, 2]
        >>> total = (1, 2)
        >>> start = (1, 4)
        >>> stop = (1, 8)
        >>> args = [pitches, total, start, stop]

    ::

        >>> notes = notetools.make_accelerating_notes_with_lilypond_multipliers(*args)

    ::

        >>> notes
        [Note("cs'8 * 113/64"), Note("d'8 * 169/128"), Note("cs'8 * 117/128")]

    ::

        >>> Voice(notes).prolated_duration
        Duration(1, 2)

    Set note pitches cyclically from `pitches`.

    Return as many interpolation values as necessary to fill the `total` duration requested.

    Interpolate durations from `start` to `stop`.

    Set note durations to `written` duration times computed interpolated multipliers.

    Return list of notes.

    .. versionchanged:: 2.0
        renamed ``construct.notes_curve()`` to
        ``notetools.make_accelerating_notes_with_lilypond_multipliers()``.
    '''
    from abjad.tools import notetools

    # initialize input arguments as durations
    total = durationtools.Duration(total)
    start = durationtools.Duration(start)
    stop = durationtools.Duration(stop)
    written = durationtools.Duration(written)

    # change mathtools input arguments to vanilla rationals
    args = [fractions.Fraction(x) for x in (total, start, stop)]
    args.append(exp)

    # calculate lilypond multipliers
    multipliers = mathtools.interpolate_divide(*args)

    # change floats to rationals
    multipliers = [durationtools.Multiplier(int(round(x * 2**10)), 2**10) for x in multipliers]

    # make notes
    result = []
    for i, multiplier in enumerate(multipliers):
        note = notetools.Note(pitches[i % len(pitches)], written)
        note.duration_multiplier = durationtools.Multiplier(multiplier / written)
        result.append(note)

    # return result
    return result



from abjad.tools.schemetools.Scheme import Scheme


class SchemeVector(Scheme):
    '''.. versionadded:: 2.0

    Abjad model of Scheme vector::

        >>> schemetools.SchemeVector(True, True, False)
        SchemeVector((True, True, False))

    Scheme vectors and Scheme vector constants differ in only their LilyPond input format.

    Scheme vectors are immutable.
    '''

    ### CLASS ATTRIBUTES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, *args):
        Scheme.__init__(self, *args, **{'quoting': "'"})



def list_durations_of_leaves_in_expr(expr):
    '''.. versionadded:: 2.0

    List prolated durations of leaves in `expr`::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 e'8 } \times 2/3 { c'8 d'8 e'8 }")

    ::

        >>> for duration in leaftools.list_durations_of_leaves_in_expr(staff):
        ...     duration
        ...
        Duration(1, 12)
        Duration(1, 12)
        Duration(1, 12)
        Duration(1, 12)
        Duration(1, 12)
        Duration(1, 12)

    Return list of durations.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import leaftools

    durations = []

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        durations.append(leaf.prolated_duration)

    return durations



from abjad import *
from abjad.tools import *


def test_Quantizer___call___01():

    milliseconds = [1500, -1000, 1000, 1000, -1000, 1000, -1000, 500]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    assert isinstance(result, Voice)
    assert result.prolated_duration == 2

    score = Score([Staff([result])])

    r'''
    \new Score <<
        \new Staff {
            \new Voice {
                {
                    \time 4/4
                    \tempo 4=60
                    c'4 ~
                    c'8
                    r8 ~
                    r8
                    c'8 ~
                    c'8
                    c'8 ~
                }
                {
                    c'8
                    r8 ~
                    r8
                    c'8 ~
                    c'8
                    r8 ~
                    r8
                    c'8
                }
            }
        }
    >>
    '''

    assert score.lilypond_format == "\\new Score <<\n\t\\new Staff {\n\t\t\\new Voice {\n\t\t\t{\n\t\t\t\t\\time 4/4\n\t\t\t\t\\tempo 4=60\n\t\t\t\tc'4 ~\n\t\t\t\tc'8\n\t\t\t\tr8 ~\n\t\t\t\tr8\n\t\t\t\tc'8 ~\n\t\t\t\tc'8\n\t\t\t\tc'8 ~\n\t\t\t}\n\t\t\t{\n\t\t\t\tc'8\n\t\t\t\tr8 ~\n\t\t\t\tr8\n\t\t\t\tc'8 ~\n\t\t\t\tc'8\n\t\t\t\tr8 ~\n\t\t\t\tr8\n\t\t\t\tc'8\n\t\t\t}\n\t\t}\n\t}\n>>"


def test_Quantizer___call___02():

    milliseconds = [250, 1000, 1000, 1000, 750]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    r'''
    \new Voice {
        {
            \time 4/4
            %%% \tempo 4=60 %%%
            c'16
            c'16 ~
            c'8 ~
            c'16
            c'16 ~
            c'8 ~
            c'16
            c'16 ~
            c'8 ~
            c'16
            c'16 ~
            c'8
        }
    }
    '''

    assert result.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 4/4\n\t\t%%% \\tempo 4=60 %%%\n\t\tc'16\n\t\tc'16 ~\n\t\tc'8 ~\n\t\tc'16\n\t\tc'16 ~\n\t\tc'8 ~\n\t\tc'16\n\t\tc'16 ~\n\t\tc'8 ~\n\t\tc'16\n\t\tc'16 ~\n\t\tc'8\n\t}\n}"


def test_Quantizer___call___03():

    q_schema = quantizationtools.BeatwiseQSchema(
        {'search_tree': quantizationtools.UnweightedSearchTree({2: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({3: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({5: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({7: None})},
        )
    milliseconds = [250, 250, 250, 250] * 4
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, q_schema=q_schema, attack_point_optimizer=attack_point_optimizer)

    r'''
    \new Voice {
        \grace {
            c'16
        }
        %%% \tempo 4=60 %%%
        c'8
        \grace {
            c'16
        }
        c'8
        \times 2/3 {
            c'8
            \grace {
                c'16
            }
            c'8
            c'8
        }
        \times 4/5 {
            c'16
            c'16
            c'16 ~
            c'16
            c'16
        }
        \times 4/7 {
            c'16 ~
            c'16
            c'16
            c'16 ~
            c'16
            c'16 ~
            c'16
        }
    }
    '''

    assert result.lilypond_format == "\\new Voice {\n\t\\grace {\n\t\tc'16\n\t}\n\t%%% \\tempo 4=60 %%%\n\tc'8\n\t\\grace {\n\t\tc'16\n\t}\n\tc'8\n\t\\times 2/3 {\n\t\tc'8\n\t\t\\grace {\n\t\t\tc'16\n\t\t}\n\t\tc'8\n\t\tc'8\n\t}\n\t\\times 4/5 {\n\t\tc'16\n\t\tc'16\n\t\tc'16 ~\n\t\tc'16\n\t\tc'16\n\t}\n\t\\times 4/7 {\n\t\tc'16 ~\n\t\tc'16\n\t\tc'16\n\t\tc'16 ~\n\t\tc'16\n\t\tc'16 ~\n\t\tc'16\n\t}\n}"



from abjad import *


def test_LayoutBlock_01():

    lb = lilypondfiletools.LayoutBlock()
    lb.indent = 0
    lb.ragged_right = True

    r'''
    \layout {
        indent = #0
        ragged-right = ##t
    }
    '''

    assert lb.lilypond_format == '\\layout {\n\tindent = #0\n\tragged-right = ##t\n}'


def test_LayoutBlock_02():

    lb = lilypondfiletools.LayoutBlock()
    m = marktools.LilyPondCommandMark('accidentalStyle modern')
    lb.append(m)

    r'''
    \layout {
        \accidentalStyle modern
    }
    '''

    assert lb.lilypond_format == '\\layout {\n\t\\accidentalStyle modern\n}'




import inspect


def get_current_function_name():
    r'''.. versionadded:: 2.10

    Get current function name::

        >>> def foo():
        ...        function_name = introspectiontools.get_current_function_name()
        ...        print 'Function name is {!r}.'.format(function_name)

    ::

        >>> foo()
        Function name is 'foo'.

    Call this function within the implementation of any ofther function.

    Returns enclosing function name as a string or else none.
    '''
    
    stack = inspect.stack()

    # per the inspect module doc page ...
    try:
        parent_frame_record = stack[1]
        parent_frame = parent_frame_record[0]
        parent_frame_info = inspect.getframeinfo(parent_frame)
        parent_frame_function_name = parent_frame_info.function
        return parent_frame_function_name
    # ... destroy frame to avoid reference cycle
    finally:
        del stack



from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_durationtools_Duration___add___01():
    '''Adding two durations returns a third duration.
    '''

    duration = Duration(1, 2) + Duration(1, 3)
    assert isinstance(duration, Duration)


def test_durationtools_Duration___add___02():
    '''Adding a duration and nonreduced fraction returns a nonreduced fraction.
    '''
    
    result = Duration(1, 4) + NonreducedFraction(2, 8)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (4, 8)

    result = NonreducedFraction(2, 8) + Duration(1, 4)
    assert isinstance(result, NonreducedFraction)
    assert result.pair == (4, 8)



def diatonic_pitch_name_to_chromatic_pitch_class_name(diatonic_pitch_name):
    '''.. versionadded:: 2.0

    Change `diatonic_pitch_name` to chromatic pitch-class name::

        >>> pitchtools.diatonic_pitch_name_to_chromatic_pitch_class_name("c''")
        'c'

    Return string.
    '''
    from abjad.tools import pitchtools

    chromatic_pitch_name = pitchtools.diatonic_pitch_name_to_chromatic_pitch_name(diatonic_pitch_name)

    return pitchtools.chromatic_pitch_name_to_chromatic_pitch_class_name(chromatic_pitch_name)



def move_component_subtree_to_right_in_immediate_parent_of_component(component):
    r'''.. versionadded:: 2.0

    Move `component` subtree to right in immediate parent of `component`::

        >>> voice = Voice("c'8 [ d'8 ] e'8 [ f'8 ]")

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            d'8 ]
            e'8 [
            f'8 ]
        }

    ::

        >>> componenttools.move_component_subtree_to_right_in_immediate_parent_of_component(
        ... voice[1])

    ::

        >>> f(voice)
        \new Voice {
            c'8 [
            e'8 ]
            d'8 [
            f'8 ]
        }

    Return none.

    .. todo:: add ``n = 1`` keyword to generalize flipped distance.

    .. todo:: make 
        ``componenttools.move_component_subtree_to_right_in_immediate_parent_of_component()`` 
        work when spanners attach to children of component:

    ::

        >>> voice = Voice(r"\times 2/3 { c'8 [ d'8 e'8 } \times 2/3 { f'8 ] g'8 a'8 }")

    ::

        >>> componenttools.move_component_subtree_to_right_in_immediate_parent_of_component(
        ... voice[0])

    ::

        >>> f(voice)
        \new Voice {
            \times 2/3 {
                f'8 ]
                g'8
                a'8
            }
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
        }

    ::

        >>> wellformednesstools.is_well_formed_component(voice)
        False

    Preserve spanners.

    .. versionchanged:: 2.0
        renamed ``componenttools.flip()`` to
        ``componenttools.move_component_subtree_to_right_in_immediate_parent_of_component()``.
    '''
    from abjad.tools import componenttools

    # swap positions in parent
    if not component.parentage.is_orphan:
        parent = component._parent
        parent_index = parent.index(component)
        try:
            next_component = parent[parent_index + 1]
        except IndexError:
            return
        parent._music[parent_index] = next_component
        parent._music[parent_index + 1] = component

    # swap positions in spanners ... tricky!
    component_spanners = {}
    for spanner in component.spanners:
        component_spanners[spanner] = spanner.index(component)
        spanner._sever_component(component)
    next_spanners = {}
    for spanner in next_component.spanners:
        next_spanners[spanner] = spanner.index(next_component)
        spanner._sever_component(next_component)
    for key, value in next_spanners.items():
        key._insert(value, component)
    for key, value in component_spanners.items():
        key._insert(value, next_component)
